from SpectrumCommon.Const.GB import *

HW_STAFF_LEVEL_FACILITY_CURR_ID = 1
HW_STAFF_LEVEL_DISTRICT_CURR_ID = 2
HW_STAFF_LEVEL_REGIONAL_CURR_ID = 3
HW_STAFF_LEVEL_NATIONAL_CURR_ID = 4
HW_NUM_STAFFING_LEVELS  = HW_STAFF_LEVEL_NATIONAL_CURR_ID

HW_STAFF_LEVEL_FACILITY_MST_ID = 1
HW_STAFF_LEVEL_DISTRICT_MST_ID = 2
HW_STAFF_LEVEL_REGIONAL_MST_ID = 3
HW_STAFF_LEVEL_NATIONAL_MST_ID = 4

HW_STAFF_LEVEL_MAP = {
    HW_STAFF_LEVEL_FACILITY_MST_ID : HW_STAFF_LEVEL_FACILITY_CURR_ID,
    HW_STAFF_LEVEL_DISTRICT_MST_ID : HW_STAFF_LEVEL_DISTRICT_CURR_ID,
    HW_STAFF_LEVEL_REGIONAL_MST_ID : HW_STAFF_LEVEL_REGIONAL_CURR_ID,
    HW_STAFF_LEVEL_NATIONAL_MST_ID : HW_STAFF_LEVEL_NATIONAL_CURR_ID,
}

HW_STAFF_LEVELS_MST_IDS = [HW_STAFF_LEVEL_FACILITY_MST_ID, HW_STAFF_LEVEL_DISTRICT_MST_ID, HW_STAFF_LEVEL_REGIONAL_MST_ID, HW_STAFF_LEVEL_NATIONAL_MST_ID]

# These work a bit differently than the standard user-entered items. Instead of adding and deleting like normal,
# there are a set number of rows and the user just enters the names on the rows. Not sure why.
HW_NUM_INSERV_TRAIN_TYPES         = 12
HW_NUM_PRESERV_TRAIN_INSTITUTIONS = 12

# Staffing methods

HW_POP_NORMS_CURR_ID           = 1
HW_FAC_STAFF_STANDARDS_CURR_ID = 2
HW_EXISTING_PLANS_CURR_ID      = 3
HW_NO_STAFF_CURR_ID            = 4
HW_NUM_STAFFING_METHODS        = 4

HW_POP_NORMS_MST_ID           = 1
HW_FAC_STAFF_STANDARDS_MST_ID = 2
HW_EXISTING_PLANS_MST_ID      = 3
HW_NO_STAFF_MST_ID            = 4

# Default ISCO medical staff types (health service providers).
# OLD -- needs to be updated if we use these anywhere.
# HR_GEN_PRIM_CARE_DOC_CURR_ID               = 1
# UH_FirstDefStaffType                       = HR_GEN_PRIM_CARE_DOC_CURR_ID
# HR_OB_GYNS_CURR_ID                         = 2
# HR_PEDIATRICIANS_CURR_ID                   = 3
# HR_OTHER_SPECIALISTS_CURR_ID               = 4
# HR_CLINIC_OFF_SURG_TECHS_CURR_ID           = 5
# HR_NURSES_CURR_ID                          = 6
# HR_MIDWIVES_CURR_ID                        = 7
# HR_ASSIST_NURSES_MIDWIVES_CURR_ID          = 8
# HR_NURSING_AIDES_CURR_ID                   = 9
# HR_LAB_TECHS_ASSIST_CURR_ID                = 10
# HR_PHARM_TECHS_ASSIST_CURR_ID              = 11
# HR_RADIO_XRAY_TECHS_CURR_ID                = 12
# HR_EMERG_MED_TECHS_CURR_ID                 = 13
# HR_COMM_HLTH_WORKERS_CURR_ID               = 14
# # Final defined default ISCO staff type. Staff types after this are placeholders.
# # for other types of staff.
# HR_NUM_DEF_ISCO_MED_STAFF_TYPES            = HR_COMM_HLTH_WORKERS_CURR_ID
# HR_OTHER_STAFF_TYPE_CURR_ID                = 15 # not necessarily a health service provider
# HR_NUM_DEF_ISCO_MED_STAFF_TYPES_PLUS_OTHER = HR_OTHER_STAFF_TYPE_CURR_ID
# HR_ADD_TO_TREAT_INPUTS_CURR_ID             = 16 # occasionally tacked on to ISCO default list to use custom staff in Intervention Costing

HW_GENERAL_MEDICAL_PRACTITIONERS_MST_ID           = 1
HW_OB_GYNS_MST_ID                                 = 2 # defunct
HW_PEDIATRICIANS_MST_ID                           = 3 # defunct
HW_SPECIAL_MEDICAL_PRACTITIONERS_MST_ID           = 4
HW_CLINIC_OFF_SURG_TECHS_MST_ID                   = 5
HW_NURSING_PROFESSIONALS_MST_ID                   = 6
HW_MIDWIFERY_PROFESSIONALS_MST_ID                 = 7
HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_MST_ID       = 8
HW_NURSING_ASSOCIATE_PROFESSIONALS_MST_ID         = 9
HW_MED_PATH_LAB_TECHS_MST_ID                      = 10
HW_PHARM_TECHS_ASSIST_MST_ID                      = 11
HW_MED_IMAGING_THERA_EQUIP_TECHS_MST_ID           = 12
HW_AMBULANCE_WORKERS_MST_ID                       = 13
HW_COMM_HLTH_WORKERS_MST_ID                       = 14
HW_OTHER_STAFF_TYPE_MST_ID                        = 15
HW_ADD_TO_TREAT_INPUTS_MST_ID                     = 16 # Special - not really a staff type
HW_DENTISTS_MST_ID                                = 25
HW_DENTAL_ASSIST_THERAPISTS_MST_ID                = 26
HW_MED_DENTAL_PROSTHETIC_REL_TECHS_MST_ID         = 27
HW_PHARMACISTS_MST_ID                             = 28
HW_PARAMED_PRACTITIONERS_MST_ID                   = 29
HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_MST_ID     = 30
HW_DIETICIANS_NUTRITIONISTS_MST_ID                = 31
HW_AUDIOLOGISTS_SPEECH_THERAPISTS_MST_ID          = 32
HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_MST_ID       = 33
HW_DISPENSING_OPTICIANS_MST_ID                    = 34
HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_MST_ID      = 35
HW_TRAD_COMP_MED_PROF_MST_ID                      = 36
HW_TRAD_COMP_MED_ASSOC_PROF_MST_ID                = 37
HW_HEALTH_CARE_ASSISTS_MST_ID                     = 38
HW_HOME_BASED_PERS_CARE_WORKERS_MST_ID            = 39
HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_MST_ID = 40
HW_PHYSIOTHERAPISTS_MST_ID                        = 41
HW_PHYSIOTHERAPY_TECHS_ASSISTS_MST_ID             = 42
HW_HEALTH_ASSOC_PROF_NOT_CLASS_MST_ID             = 43
HW_PSYCHOLOGISTS_MST_ID                           = 44
HW_SOCIAL_WORK_COUNSELLING_PROF_MST_ID            = 45
HW_SOCIAL_WORK_ASSOC_PROF_MST_ID                  = 46

HW_GENERAL_MEDICAL_PRACTITIONERS_CURR_ID           = 1
HW_SPECIAL_MEDICAL_PRACTITIONERS_CURR_ID           = 2
HW_CLINIC_OFF_SURG_TECHS_CURR_ID                   = 3
HW_NURSING_PROFESSIONALS_CURR_ID                   = 4
HW_MIDWIFERY_PROFESSIONALS_CURR_ID                 = 5
HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_CURR_ID       = 6
HW_NURSING_ASSOCIATE_PROFESSIONALS_CURR_ID         = 7
HW_MED_PATH_LAB_TECHS_CURR_ID                      = 8
HW_PHARM_TECHS_ASSIST_CURR_ID                      = 9
HW_MED_IMAGING_THERA_EQUIP_TECHS_CURR_ID           = 10
HW_AMBULANCE_WORKERS_CURR_ID                       = 11
HW_COMM_HLTH_WORKERS_CURR_ID                       = 12
HW_OTHER_STAFF_TYPE_CURR_ID                        = 13
HW_DENTISTS_CURR_ID                                = 14
HW_DENTAL_ASSIST_THERAPISTS_CURR_ID                = 15
HW_MED_DENTAL_PROSTHETIC_REL_TECHS_CURR_ID         = 16
HW_PHARMACISTS_CURR_ID                             = 17
HW_PARAMED_PRACTITIONERS_CURR_ID                   = 18
HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_CURR_ID     = 19
HW_DIETICIANS_NUTRITIONISTS_CURR_ID                = 20
HW_AUDIOLOGISTS_SPEECH_THERAPISTS_CURR_ID          = 21
HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_CURR_ID       = 22
HW_DISPENSING_OPTICIANS_CURR_ID                    = 23
HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_CURR_IDC     = 24
HW_TRAD_COMP_MED_PROF_CURR_ID                      = 25
HW_TRAD_COMP_MED_ASSOC_PROF_CURR_ID                = 26
HW_HEALTH_CARE_ASSISTS_CURR_ID                     = 27
HW_HOME_BASED_PERS_CARE_WORKERS_CURR_ID            = 28
HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_CURR_ID = 29
HW_PHYSIOTHERAPISTS_CURR_ID                        = 30
HW_PHYSIOTHERAPY_TECHS_ASSISTS_CURR_ID             = 31
HW_HEALTH_ASSOC_PROF_NOT_CLASS_CURR_ID             = 32
HW_PSYCHOLOGISTS_CURR_ID                           = 33
HW_SOCIAL_WORK_COUNSELLING_PROF_CURR_ID            = 34
HW_SOCIAL_WORK_ASSOC_PROF_CURR_ID                  = 35
HW_NUM_STATIC_STAFF_TYPES                          = HW_SOCIAL_WORK_ASSOC_PROF_CURR_ID

# Use this to refer to all default and custom staff combined.
UH_AllDefaultAndCustomStaff = 10000

# Default health management and support (HMS) personnel.
# OLD -- needs to be updated if we use these anywhere.
# HR_HLTH_SERV_MANAGERS_CURR_ID        = 1
# HR_FirstDefHMSStaffType_CURR_ID      = HR_HLTH_SERV_MANAGERS_CURR_ID
# HR_HLTH_MGMT_PERSONNEL_CURR_ID       = 2
# HR_NON_HLTH_PROFESSIONALS_CURR_ID    = 3
# HR_MED_SECRETARIES_CURR_ID           = 4
# HR_NON_HLTH_TECHS_ASSOC_PROF_CURR_ID = 5
# HR_CLERICAL_SUPP_WORKERS_CURR_ID     = 6
# HR_PLANT_MACH_OPER_ASSEMB_CURR_ID    = 7
# HR_ELEMENTARY_OCCUPATIONS_CURR_ID    = 8
# HR_NUM_DEF_HMS_STAFF_TYPES           = HR_ELEMENTARY_OCCUPATIONS_CURR_ID

# These are added to the same staff list as the medical staff, so make it so their master IDs don't overlap.
HW_HLTH_SERV_MANAGERS_MST_ID                  = 17
HW_HLTH_PROF_NOT_ELSEWHERE_CLASSIFIED_MST_ID  = 18
HW_NON_HLTH_PROFESSIONALS_MST_ID              = 19
HW_MED_SECRETARIES_MST_ID                     = 20
HW_NON_HLTH_TECHS_ASSOC_PROF_MST_ID           = 21
HW_CLERICAL_SUPP_WORKERS_MST_ID               = 22
HW_PLANT_MACH_OPER_ASSEMB_MST_ID              = 23
HW_ELEMENTARY_OCCUPATIONS_MST_ID              = 24
HW_MED_REC_HEALTH_INFO_TECHS_MST_ID           = 47
HW_BIOMED_ENGINEER_MST_ID                     = 48
HW_AGED_CARE_SERVICE_MANAGERS_MST_ID          = 49

# Maximum number of staff types the user can enter.
HW_MAX_STAFF_TYPES = 500

# Constants for which staff division a staff type goes under.
HW_HLTH_SERV_PROVIDER_CURR_ID  = 1
HW_HLTH_MGMT_SUPP_PERS_CURR_ID = 2
HW_NUM_STAFF_DIVISIONS         = HW_HLTH_MGMT_SUPP_PERS_CURR_ID

HW_HLTH_SERV_PROVIDER_MST_ID  = 1
HW_HLTH_MGMT_SUPP_PERS_MST_ID = 2

# Compensation types
HW_SALARIES_CURR_ID   = 1
HW_BENEFITS_CURR_ID   = 2
HW_INCENTIVES_CURR_ID = 3
HW_NUM_COMPENS_TYPES  = HW_INCENTIVES_CURR_ID

HW_SALARIES_MST_ID   = 1
HW_BENEFITS_MST_ID   = 2
HW_INCENTIVES_MST_ID = 3

HW_COMPENS_TYPE_MAP = {
    HW_SALARIES_MST_ID   : HW_SALARIES_CURR_ID,
    HW_BENEFITS_MST_ID   : HW_BENEFITS_CURR_ID,
    HW_INCENTIVES_MST_ID : HW_INCENTIVES_CURR_ID,
}

HW_COMPENS_TYPE_MST_IDS = [HW_SALARIES_MST_ID, HW_BENEFITS_MST_ID, HW_INCENTIVES_MST_ID]

HW_POP_FACTOR = 10000

# Database latest versions
HW_STAFF_SALARIES_DB_CURR_VERSION = 'V1'

# Database names (also the names of the sheets in HWModData.xlsx)
HW_STAFF_SALARIES_DB_NAME = 'StaffSalariesDB'

# Database directories for data by country. Files in these folders are named using the 3 letter iso country codes instead of the database names above.
HW_STAFF_SALARIES_DB_DIR = r'StaffSalaries\countries'



# Input ModVars

HW_VERSION_TAG_V1                                = '<HR_Version_V1>'
HW_DATA_CHANGED_TAG_V1                           = '<HR_DataChanged_V1>'
HW_PROJ_VALID_TAG_V1                             = '<HR_ProjectionValid_V1>'
HW_FILE_NAME_TAG_V1                              = '<HR_FileName_V1>'                        
HW_PROJ_VALID_DATE_TAG_V1                        = '<HR_ProjectionValidDate_V1>'   
HW_COST_INSERV_TRAIN_BY_PERC_SALARY_TAG_V1       = '<HR_CostInservTrainByPercSalary_V1>'
HW_SHOW_PRESERV_TRAIN_BY_INSTITUTION_TAG_V1      = '<HR_ShowPreservTrainByInstitution_V1>'
# no longer used?  HR_SHOW_ALL_YEARS_SCALE_UP_POL_ANA_TAG_V1   = '<HR_ShowAllYearsScaleUpPolAna_V1>'
HW_TARG_ENROLLMENT_VISITED_TAG_V1                = '<HR_TargEnrollmentVisited_V1>'                  
HW_TARG_GRADUATION_VISITED_TAG_V1                = '<HR_TargGraduationVisited_V1>'      
HW_STAFF_TYPE_RECORDS_TAG_V1                     = '<HR_StaffRecords_V1>'             
HW_INSERV_TRAIN_TYPE_RECORDS_TAG_V1              = '<HR_InserviceTrainingTypeRecords_V1>'
    #HR_INSERV_TRAIN_STAFF_MAP_TAG_V1              = '<HR_InservTrainStaffMap_V1>'    
    #HR_trainTypeNames            
    #HR_costPerStaffTrained 
    #HR_numStaffToBeTrained   
# no longer used? HR_INCOME_ELASTICITY_TAG_V1                = '<HR_IncomeElasticity_V1>'
# no longer used? HR_ECONOMIC_GROWTH_TAG_V1                  = '<HR_EconomicGrowth_V1>'   
# no longer used? HR_INCR_PUBLIC_SECTOR_MATRIC_INCENT_TAG_V1 = '<HR_IncrPublicSectorMatricIncentives_V1>'   
# no longer used? HR_IMP_PUBLIC_SECTOR_MATRIC_INCENT_TAG_V1  = '<HR_ImpPublicSectorMatricIncentives_V1>'
HW_TARG_SCALE_UP_VISITED_TAG_V1                  = '<HR_TargetScaleUpVisited_V1>'  
HW_TARG_RETENT_INCENT_LEVELS_VISITED_TAG_V1      = '<HR_TargetRetentIncentLevelsVisited_V1>'        
HW_ATTRITION_TYPE_RECORDS_TAG_V1                 = '<HR_AttritionTypeRecords_V1>'
HW_RETENTION_INCENTIVE_RECORDS_TAG_V1            = '<HR_RetentionIncentiveRecords_V1>'
HW_RECRUITMENT_INCENTIVE_RECORDS_TAG_V1          = '<HR_RecruitmentIncentiveRecords_V1>'
# no longer used? HR_PRIVATE_SECTOR_HRH_TAG_V1            = '<HR_PrivateSectorHRH_V1>'
HW_PRESERV_ENROLLMENT_TAG_V1                     = '<HR_PreservEnrollment_V1>'
HW_PROJECTED_GRADUATION_TAG_V1                   = '<HR_ProjectedGraduation_V1>'                    
HW_TOTAL_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG_V1 = '<HR_TotalInservTrainCostsByTrainType_V1>'
HW_CURR_GRAD_PERC_CURR_ENROLL_TAG_V1             = '<HR_CurrGraduationAsPercCurrEnrollment_V1>'
HW_ATTRITION_RATE_TAG_V1                         = '<HR_AttritionRate_V1>'
HW_COST_PRESERV_TRAIN_TAG_V1                     = '<HR_CostPreservTrain_V1>'  
HW_TOTAL_INSERV_TRAIN_COSTS_BY_STAFF_TAG_V1      = '<HR_TotalInservTrainCostsByStaffType_V1>'     
HW_STAFF_NEEDING_TO_BE_HIRED_TAG_V1              = '<HR_StaffNeedingToBeHired_V1>'
HW_TOTAL_STAFF_TAG_V1                            = '<HR_TotalStaff_V1>'               
HW_TOTAL_COMPENSATION_TAG_V1                     = '<HR_TotalCompensation_V1>'
HW_PROG_MGMT_DATA_TAG_V1                         = '<HR_ProgMgmtData_V1>'

# Output / Result ModVars

HW_RES_TOTAL_STAFF_TAG_V1                        = '<HR_ResTotalStaff_V1>'
HW_RES_STAFF_PER_10K_POP_TAG_V1                  = '<HR_ResStaffPer10KPop_V1>'
HW_RES_TOTAL_COMPENSATION_TAG_V1                 = '<HR_ResTotalCompensation_V1>'
HW_RES_STAFF_NEEDING_TO_BE_HIRED_TAG_V1          = '<HR_ResStaffNeedingToBeHired_V1>'
HW_RES_ATTRITION_RATE_TAG_V1                     = '<HR_ResAttritionRate_V1>'
HW_RES_INSERV_TRAIN_COSTS_BY_STAFF_TAG_V1        = '<HR_ResInservTrainCostsByStaff_V1>'
HW_RES_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG_V1   = '<HR_ResInservTrainCostsByTrainType_V1>'
HW_RES_PRESERV_TRAIN_COSTS_TAG_V1                = '<HR_ResPreservTrainCosts_V1>'

# Meta ModVars. The values inside these do not change. 

HW_STATIC_STAFF_LIST_META_TAG_V1 = '<HR_DefaultStaffList_Meta_V1>'



import SpectrumCommon.Const.GB.GBConst as gbc
import SpectrumCommon.Const.LG.LGModVarFields as lgmvf

#######################################################################################################
#                                                                                                     #
#                                      Supply chain levels                                            #
#                                                                                                     #
#######################################################################################################


def get_LG_num_supply_chain_levels(supply_chain_level_records: list):
    return len(supply_chain_level_records)


def get_LG_supply_chain_level_rec(supply_chain_level_records: list, ID: int):
    supply_chain_level_recs = [
        supply_chain_level_rec
        for supply_chain_level_rec in supply_chain_level_records
        if supply_chain_level_rec[lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID] == ID
    ]
    return supply_chain_level_recs[0] if len(supply_chain_level_recs) > 0 else {}


def get_LG_supply_chain_level_idx(supply_chain_level_records: list, ID: str):
    indices = [
        idx
        for (idx, supply_chain_level_rec) in enumerate(supply_chain_level_records)
        if supply_chain_level_rec[lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID] == ID
    ]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND


###


def get_LG_supply_chain_level_name(supply_chain_level_rec: dict):
    return supply_chain_level_rec[lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME]


def set_LG_supply_chain_level_name(supply_chain_level_rec: dict, value: str):
    supply_chain_level_rec[lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME] = value


###


def get_LG_supply_chain_level_custom(supply_chain_level_rec: dict):
    return supply_chain_level_rec[lgmvf.LG_SUPPLY_CHAIN_LEVEL_CUSTOM]


def set_LG_supply_chain_level_custom(supply_chain_level_rec: dict, value: bool):
    supply_chain_level_rec[lgmvf.LG_SUPPLY_CHAIN_LEVEL_CUSTOM] = value


#######################################################################################################
#                                                                                                     #
#                                      Warehouses                                                     #
#                                                                                                     #
#######################################################################################################


def get_LG_warehouse_idx(warehouse_records: list, ID: str):
    indices = [
        idx for (idx, warehouse_rec) in enumerate(warehouse_records) if warehouse_rec[lgmvf.LG_WAREHOUSE_ID] == ID
    ]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND


def get_LG_warehouse_rec(warehouse_records: list, ID: str):
    warehouse_recs = [
        warehouse_rec for warehouse_rec in warehouse_records if warehouse_rec[lgmvf.LG_WAREHOUSE_ID] == ID
    ]
    return warehouse_recs[0] if len(warehouse_recs) > 0 else {}


def get_LG_warehouse_records_for_disag_level(warehouse_records: list, disag_level: int):
    return [
        warehouse_record
        for warehouse_record in warehouse_records
        if warehouse_record.get(lgmvf.LG_WAREHOUSE_SUPPLY_CHAIN_LEVEL_ID) == disag_level
    ]


def get_LG_warehouse_records_up_to_disag_level(warehouse_records: list, disag_level: int):
    return [
        warehouse_record
        for warehouse_record in warehouse_records
        if warehouse_record.get(lgmvf.LG_WAREHOUSE_SUPPLY_CHAIN_LEVEL_ID) <= disag_level
    ]


def get_LG_warehouse_type(warehouse_record: dict = {}):
    return warehouse_record.get(lgmvf.LG_WAREHOUSE_TYPE)


def set_LG_warehouse_name(warehouse_rec: dict, value: str):
    warehouse_rec[lgmvf.LG_WAREHOUSE_NAME] = value


def get_LG_warehouse_supply_chain_level_ID(warehouse_rec: dict):
    return warehouse_rec[lgmvf.LG_WAREHOUSE_SUPPLY_CHAIN_LEVEL_ID]


def get_LG_warehouse_ID(warehouse_rec: dict):
    return warehouse_rec[lgmvf.LG_WAREHOUSE_ID]


#######################################################################################################
#                                                                                                     #
#                                      Vehicle types                                                  #
#                                                                                                     #
#######################################################################################################


def get_LG_num_vehicle_types(vehicle_type_records: list):
    return len(vehicle_type_records)


def get_LG_vehicle_type_rec(vehicle_type_records: list, ID: str):
    for vehicle_type_rec in vehicle_type_records:
        if vehicle_type_rec.get(lgmvf.LG_VEHICLE_TYPE_ID) == ID:
            return vehicle_type_rec
    return {}


def get_LG_vehicle_type_idx(vehicle_type_records: list, ID: str):
    indices = [
        idx
        for (idx, vehicle_type_rec) in enumerate(vehicle_type_records)
        if vehicle_type_rec[lgmvf.LG_VEHICLE_TYPE_ID] == ID
    ]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND


###


def get_LG_vehicle_type_name(vehicle_type_rec: dict):
    return vehicle_type_rec[lgmvf.LG_VEHICLE_TYPE_NAME]


def set_LG_vehicle_type_name(vehicle_type_rec: dict, value: str):
    vehicle_type_rec[lgmvf.LG_VEHICLE_TYPE_NAME] = value


###


def get_LG_vehicle_type_custom(vehicle_type_rec: dict):
    return vehicle_type_rec[lgmvf.LG_VEHICLE_TYPE_CUSTOM]


def set_LG_vehicle_type_custom(vehicle_type_rec: dict, value: bool):
    vehicle_type_rec[lgmvf.LG_VEHICLE_TYPE_CUSTOM] = value


#######################################################################################################
#                                                                                                     #
#                                      Worker types                                                  #
#                                                                                                     #
#######################################################################################################


def get_LG_num_worker_types(worker_type_records: list):
    return len(worker_type_records)


def get_LG_worker_type_rec(worker_type_records: list, ID: str):
    worker_type_recs = [
        worker_type_rec for worker_type_rec in worker_type_records if worker_type_rec[lgmvf.LG_WORKER_TYPE_ID] == ID
    ]
    return worker_type_recs[0] if len(worker_type_recs) > 0 else {}


def get_LG_worker_type_idx(worker_type_records: list, ID: str):
    indices = [
        idx
        for (idx, worker_type_rec) in enumerate(worker_type_records)
        if worker_type_rec[lgmvf.LG_WORKER_TYPE_ID] == ID
    ]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND


###


def get_LG_worker_type_name(worker_type_rec: dict):
    return worker_type_rec[lgmvf.LG_WORKER_TYPE_NAME]


def set_LG_worker_type_name(worker_type_rec: dict, value: str):
    worker_type_rec[lgmvf.LG_WORKER_TYPE_NAME] = value


###


def get_LG_worker_type_custom(worker_type_rec: dict):
    return worker_type_rec[lgmvf.LG_WORKER_TYPE_CUSTOM]


def set_LG_worker_type_custom(worker_type_rec: dict, value: bool):
    worker_type_rec[lgmvf.LG_WORKER_TYPE_CUSTOM] = value


#######################################################################################################
#                                                                                                     #
#                                      Cold chain records                                             #
#                                                                                                     #
#######################################################################################################


def get_LG_cold_chain_rec(cold_chain_records: list, ID: str):
    cold_chain_recs = [
        cold_chain_rec for cold_chain_rec in cold_chain_records if cold_chain_rec[lgmvf.LG_COLD_CHAIN_ID] == ID
    ]
    return cold_chain_recs[0] if len(cold_chain_recs) > 0 else {}


def get_LG_cold_chain_rec_idx(cold_chain_records: list, ID: str):
    indices = [
        idx for (idx, cold_chain_rec) in enumerate(cold_chain_records) if cold_chain_rec[lgmvf.LG_COLD_CHAIN_ID] == ID
    ]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND


def set_LG_cold_chain_rec_name(cold_chain_rec: dict, value: str):
    cold_chain_rec[lgmvf.LG_COLD_CHAIN_NAME] = value


def set_LG_cold_chain_rec_custom(cold_chain_rec: dict, value: bool):
    cold_chain_rec[lgmvf.LG_COLD_CHAIN_CUSTOM] = value


def set_LG_cold_chain_rec_capacity(cold_chain_rec: dict, value: int):
    cold_chain_rec[lgmvf.LG_COLD_CHAIN_CAPACITY] = value


def set_LG_cold_chain_rec_unit_cost(cold_chain_rec: dict, value: int):
    cold_chain_rec[lgmvf.LG_COLD_CHAIN_UNIT_COST] = value


#######################################################################################################
#                                                                                                     #
#                                      Third party logistics contracts                                #
#                                                                                                     #
#######################################################################################################


def get_LG_num_third_party_log_contracts(third_party_log_contract_records: list):
    return len(third_party_log_contract_records)


def get_LG_third_party_log_contract_rec(third_party_log_contract_records: list, ID: str):
    third_party_log_contract_recs = [
        third_party_log_contract_rec
        for third_party_log_contract_rec in third_party_log_contract_records
        if third_party_log_contract_rec[lgmvf.LG_TPL_CONTRACT_ID] == ID
    ]
    return third_party_log_contract_recs[0] if len(third_party_log_contract_recs) > 0 else {}


def get_LG_third_party_log_contract_idx(third_party_log_contract_records: list, ID: str):
    indices = [
        idx
        for (idx, third_party_log_contract_rec) in enumerate(third_party_log_contract_records)
        if third_party_log_contract_rec[lgmvf.LG_F_THIRD_PARTY_LOG_CONTRACT_ID] == ID
    ]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND


###


def get_LG_third_party_log_contract_name(third_party_log_contract_rec: dict):
    return third_party_log_contract_rec[lgmvf.LG_TPL_CONTRACT_NAME]


def set_LG_third_party_log_contract_name(third_party_log_contract_rec: dict, value: str):
    third_party_log_contract_rec[lgmvf.LG_TPL_CONTRACT_NAME] = value


###


def get_LG_third_party_log_contract_custom(third_party_log_contract_rec: dict):
    return third_party_log_contract_rec[lgmvf.LG_TPL_CONTRACT_CUSTOM]


def set_LG_third_party_log_contract_custom(third_party_log_contract_rec: dict, value: bool):
    third_party_log_contract_rec[lgmvf.LG_TPL_CONTRACT_CUSTOM] = value



import numpy as np
from .make_model3 import make_model3

#from make_model import make_model
from .goveqs_scaleup import goveqs_scaleup
from .ode15s import ode15s, create_steps

from .InterventionScenario import InterventionScenario

from scipy import interpolate
import collections.abc


def RunGlobalEpiModelb(PARS,IntvnScn, bDoBaseline=False): # function [epi_outputs]=RunGlobalEpiModelb(PARS,IntvnScn)

    epi_outputs=[] #epi_outputs=[];

    #xsto2 = PARS[0] #xsto2=PARS{1};
    #outsto2 = PARS[1] #outsto2=PARS{2};
    
    #xsto = xsto2
    #M , I = outsto2.max(), outsto2.argmax() #[M,I] = max(outsto2);

    xi = PARS[2] #xi=PARS{3};
    obj= PARS[3] #obj=PARS{4};

    i0 = PARS[4] #i0=PARS{5};
    s0 = PARS[5] #s0=PARS{6};
    gps0=PARS[6] #gps0=PARS{7};
    sel0=PARS[7] #sel0=PARS{8};
    agg0=PARS[8] #agg0=PARS{9};
    r = PARS[9] #r=PARS{10};
    p = PARS[10] #p=PARS{11};
    prm=PARS[11] #prm=PARS{12};
    init0 = PARS[12] #init0

    #%load Model_setup;
    #xs = xsto[I,:] #xs = xsto(I,:);  % Most probable parameter set #Commented out on June 09 2024 after Carel suggested we stop passing xs as a parameter

    #% % --- Get the Regional notifications
    tend1   = IntvnScn['year1'] #tend1   = IntvnScn.year1;%2024;  % Intervention starts except vaccination  
    tend1b  = IntvnScn['year2'] #tend1b  = IntvnScn.year2;%2028;  % Vaccination starts
    tend2   = IntvnScn['year3'] #tend2   = IntvnScn.year3;%2036;  % End date for simulation

    #% --- Do the simulations 
    opts = {'NonNegative': list(range(1, i0['nstates'] + 1)), 'rtol': 1e-10, 'atol': 1e-10, 'max_step': 1/64}#opts = odeset('NonNegative',[1:i0.nstates],'Refine',64,'AbsTol',1e-10,'RelTol',1e-10);

    r0 = r.copy(); p0 = p.copy()

    inct   = []
    inch1  = []
    mdr    = []
    morth0 = []
    morth1 = []
    noti   = []
    notimdr= []
    prevtb = []


    #[r,p] = alloc_parameters(xs,prm['r'],prm['p'],xi) #[r,p] = alloc_parameters(xs,prm.r,prm.p,xi);
    #% Get the initial conditions
    init = init0.copy()
    if init == []: #Check if init0 was provided
        [out, aux] = obj() #[out, aux] = obj(xs);
        init = aux['soln'][-1,:] #init = aux.soln(end,1:end);    

    #init_matlab = np.loadtxt("C:\\Users\\GuyMahiane\\Documents\\GM_work\\Carel\\20231010_TB_IHT\\2024-07-11\\NGA_final_vaccine2f\\init0.txt")
    #init = init_matlab

    #% Extend initial condition for the full model with vaccine compartments  
    inds1 = np.intersect1d(s0['U'], s0['v0']) #inds1 = intersect(s0.U, s0.v0);
    inds2 = np.intersect1d([s0['Lf'], s0['Ls']], s0['v0']) #inds2 = intersect([s0.Lf, s0.Ls], s0.v0);
    inds3 = np.concatenate([s0['Isc'], s0['I'], s0['E'], s0['Rlo'], s0['Rhi'], s0['R']]) #inds3 = [s0.Isc, s0.I, s0.E, s0.Rlo, s0.Rhi, s0.R];
    inds4 = np.concatenate([s0['Dx'],s0['Tx'],s0['Tx2']]) #inds4 = [s0.Dx,s0.Tx,s0.Tx2];
    inds5 =   s0['Tx2'][-1] + np.array(range(1,10)) #inds5 =  s0.Tx2(end)+[1:9] #inds5 =  s0.Tx2(end)+[1:9];  % Auxiliary terms
    ind = np.concatenate([inds1, inds2, inds3,inds4,inds5]) #ind = [inds1, inds2, inds3,inds4,inds5];
   

    #% To allocate results from a model without vaccine to input vector for full model with vaccine
    init0 = np.zeros((1,i0['nx'])) #init0 = zeros(1,i0.nx);
    init0[0][ind] = init #init0(ind) = init;   

    tref = np.arange(2022, tend2 + 1, 1) #tref = [2022:1:tend2];

    # % --- Now simulate without intervention (baseline)
    p['VE'] = np.array([0.7, 0.7]) #Carel asked to add this on 2024-10-18
    #p['VE'] = np.array([0.0, 0.0]) 
    M0 = make_model3(p, r, i0, s0, gps0, prm)#M0 = make_model3(p, r, i0, s0, gps0, prm); 
    #**** Apparently, this was changed at some point. Guy is making a change on 2024-07-24
    #prm['growth'] = prm['gth'][2] #prm.growth = prm.gth(3);
    prm['growth'] = prm['gth'][3] #prm.growth = prm.gth(4);
    #****End 

    #% Find the initial condition at the starting point for the interventions
    #geq = @(t,in) goveqs_scaleup(t, in, M0, M0, [2022 2023], i0, prm, sel0, agg0);
    #[ta, solna] = ode15s(geq, [2022 tend1], init0, odeset('NonNegative',[1:i0.nstates]));
    #initb = solna(end,:);
    ta, solna = ode15s(goveqs_scaleup, init0[0], create_steps(2022, tend1[0], 1/2), args=(r, M0, M0, [2022, 2023], i0, prm, sel0, agg0))
    initb = solna[-1,:]

    #% --- Next simulate with interventions
    #%r1 = r; p1 = p;
    #%Set intervention to model parameters
    r1 = r.copy()
    p1 = p.copy()
    
    if not bDoBaseline:
        [p1,r1] = InterventionScenario(p,r,IntvnScn) #[p1,r1]=SetInterventionScenario(p,r,IntvnScn);

    if isinstance(r1['Tx2'], (collections.abc.Sequence, np.ndarray)) :
        if len(r1['Tx2'])==1:
            r1['Tx2'] = r1['Tx2'][0]

    for cur_x in ['cure', 'cure2']:
        if cur_x in list(p1):
            if len(p1[cur_x])>=1:
                for ii in range(len(p1[cur_x])):
                    if isinstance(p1[cur_x][ii], (collections.abc.Sequence, np.ndarray)):
                        p1[cur_x][ii] = p1[cur_x][ii][0]  

    M1 = make_model3(p1, r1, i0, s0, gps0, prm) #M1 = make_model3(p1, r1, i0, s0, gps0, prm);

    #% p1
    #% r1
    #% pause

    #geq = @(t,in) goveqs_scaleup(t, in, M0, M1, [tend1 tend1b], i0, prm, sel0, agg0);
    #[tb, solnb] = ode15s(geq, [tend1 tend2], initb, opts);

    tb, solnb = ode15s(goveqs_scaleup, initb, create_steps(tend1[0], tend2[0], 1/2), args=(r1, M0, M1, np.array([tend1[0], tend1b[0]]), i0, prm, sel0, agg0))
    initb = solnb[-1,:]

    soln  = np.concatenate([solna, solnb[1:,:]]) #soln  = [solna; solnb(2:end,:)];
    t     = np.concatenate([ta,    tb[1:]]) #t     = [ta;    tb(2:end)];
    soln8 = np.zeros((len(tref),np.shape(soln)[1])) #soln8 = interp1(t, soln, tref); 
    for jj in range(np.shape(soln)[1]):#range(i['nx']):
        temp_fn_sol = interpolate.interp1d(t, soln[:,jj],fill_value="extrapolate") 
        soln8[:,jj] = [temp_fn_sol(ti) for ti in tref]


    allmat = soln8 
    dallmat = np.diff(allmat,n=1,axis=0) #dallmat = diff(allmat,[],1);

    #pop = np.squeeze(np.sum(dallmat[:,np.concatenate([s0['U'],s0['Lf'],s0['Ls'],s0['prevalent'],s0['Rlo'],s0['Rhi'],s0['R']])],1))#pop(:,:) = squeeze(sum(allmat(1:end-1,[s0.U,s0.Lf,s0.Ls,s0.prevalent,s0.Rlo,s0.Rhi,s0.R],:),2)); %Total population
    #pop = np.squeeze(np.sum(allmat[:,:],1))[:-1]#pop(:,:) = squeeze(sum(allmat(1:end-1,[s0.U,s0.Lf,s0.Ls,s0.prevalent,s0.Rlo,s0.Rhi,s0.R],:),2)); %Total population
    pop = np.sum(soln[0:(np.shape(allmat)[0]-1),0:(i0['nstates']-1)],1)

    inct = np.squeeze(dallmat[:,i0['aux']['inc'][0]])/pop*1e5 #inct(:,:) = squeeze(dallmat(:,i0.aux.inc(1),:))*1e5;    % Total incidence
    inch1 = np.squeeze(dallmat[:,i0['aux']['inc'][1]])/pop*1e5 #inch1(:,:) = squeeze(dallmat(:,i0.aux.inc(2),:))*1e5;   % HIV incidence
    mdr = np.squeeze(dallmat[:,i0['aux']['inc'][2]])/pop*1e5 #mdr(:,:) = squeeze(dallmat(:,i0.aux.inc(3),:))*1e5;     % MDR incidence
    morth0 = np.squeeze(dallmat[:,i0['aux']['mort'][0]])/pop*1e5 #; % hiv-negative mortality
    morth1 = np.squeeze(dallmat[:,i0['aux']['mort'][1]])/pop*1e5 #morth1(:,:) = squeeze(dallmat(:,i0.aux.mort(2),:))*1e5; % hiv-positive mortality
    noti = np.squeeze(np.sum(dallmat[:,i0['aux']['noti']],1))/pop*1e5 #noti(:,:) = squeeze(sum(dallmat(:,i0.aux.noti,:),2))*1e5; % Total notification
    notimdr = np.squeeze(dallmat[:,i0['aux']['noti2']])/pop*1e5 #notimdr(:,:) = squeeze(dallmat(:,i0.aux.noti2,:))*1e5;  % MDR Treatment initiation
    prevtb = np.squeeze(np.sum(allmat[1:,s0['prevalent']],1))*1e5 #;%TB prevalence #prevtb(:,: ) = squeeze(sum(allmat(2:end,s0.prevalent,:),2))*1e5;%TB prevalence
  
    
    #pdb.set_trace() 

    epi_outputs = []
    epi_outputs.append(inct)#epi_outputs[0] = inct
    epi_outputs.append(inch1) #epi_outputs[1] = inch1
    epi_outputs.append(mdr) #epi_outputs[2] = mdr
    epi_outputs.append(morth0) #epi_outputs[3] = morth0
    epi_outputs.append(morth1) #epi_outputs[4] = morth1
    epi_outputs.append(noti) #epi_outputs[5] = noti
    epi_outputs.append(notimdr) #epi_outputs[6] = notimdr
    epi_outputs.append(prevtb)
    
    return(epi_outputs)

#end




# File Version
BG_FILE_VERSION_1_0 = 1.0

# Database latest versions
BG_BUDGET_DB_CURR_VERSION            = 'V3'  
BG_BUDGET_CAT_DB_CURR_VERSION        = 'V3'
BG_INDICATOR_MAPPING_DB_CURR_VERSION = 'V2'
BG_FUND_FW_DB_CURR_VERSION           = 'V3'
BG_FUND_SOURCE_DB_CURR_VERSION       = 'V3'

BG_BUDGET_DB_NAME            = 'BudgetDB'  
BG_BUDGET_CAT_DB_NAME        = 'BudgetCategoryDB'
BG_INDICATOR_MAPPING_DB_NAME = 'IndicatorMappingDB'
BG_FUND_FW_DB_NAME           = 'FundingFrameworkDB'
BG_FUND_SOURCE_DB_NAME       = 'FundingSourceDB'

# Budget category master IDs. (cast as str)
BG_BC_DISTRIB_MULTI_CATS_MST_ID = 0
BG_BC_UNALLOCATED_MST_ID        = 1

# Funding source master IDs. (cast as str)
BG_FS_DISTRIB_MULTI_SOURCES_MST_ID = 0
BG_FS_UNALLOCATED_MST_ID           = 1

# Quick mapping targets
BG_QUICK_MAP_BUDGET_CATS_MST_ID  = 1
BG_QUICK_MAP_FUND_SOURCES_MST_ID = 2

BG_QUICK_MAP_TARGET_MST_IDS = [BG_QUICK_MAP_BUDGET_CATS_MST_ID, BG_QUICK_MAP_FUND_SOURCES_MST_ID]

#   *************************   IC indicator types   **************************

#  Note that BG_IC_DRUG_SUPPLY_IND_MST_ID and BG_IC_INTERV_IND_MST_ID are really the
#  same thing (intervention drug and supply costs), but the former is shown
#  by drug and supply and the latter is shown by intervention (as the labor
#  and visit costs are).

BG_IC_DRUG_SUPPLY_IND_MST_ID        = 1 # Drug/supply costs by drug/supply; by delivery channel; IHT, TB
BG_IC_INTERV_IND_MST_ID             = 2 # Drug/supply costs by intervention; by delivery channel; IHT, TB
BG_IC_LABOR_COSTS_IND_MST_ID        = 3 # Disease-specific costing in IHT, TB app; by delivery channel
BG_IC_VISIT_COSTS_IND_MST_ID        = 4 # Disease-specific costing in IHT, TB app; by delivery channel
BG_IC_LOG_SUPPLY_CHAIN_COSTS_MST_ID = 5 # TB app; not by delivery channel
BG_IC_WASTAGE_COSTS_MST_ID          = 6 # TB app; not by delivery channel
BG_IC_NUM_IND_TYPES                 = 6

# All possible IC indicators

BG_IC_IND_TYPE_MST_IDS = [
    BG_IC_DRUG_SUPPLY_IND_MST_ID, 
    BG_IC_INTERV_IND_MST_ID, 
    BG_IC_LABOR_COSTS_IND_MST_ID, 
    BG_IC_VISIT_COSTS_IND_MST_ID, 
    BG_IC_LOG_SUPPLY_CHAIN_COSTS_MST_ID,
    BG_IC_WASTAGE_COSTS_MST_ID,
]

# All IC indicators than can be displayed by intervention
BG_IC_INTERV_IND_SET = [
    BG_IC_INTERV_IND_MST_ID, 
    BG_IC_LABOR_COSTS_IND_MST_ID, 
    BG_IC_VISIT_COSTS_IND_MST_ID
]

# All IC indicators that can be displayed by drug/supply
BG_IC_DRUG_SUPPLY_IND_SET = [
    BG_IC_DRUG_SUPPLY_IND_MST_ID, 
    BG_IC_LOG_SUPPLY_CHAIN_COSTS_MST_ID, 
    BG_IC_WASTAGE_COSTS_MST_ID
]

# All IC indicators that vary by delivery channel
BG_IC_DELIV_CHAN_IND_SET = [
    BG_IC_DRUG_SUPPLY_IND_MST_ID, 
    BG_IC_INTERV_IND_MST_ID,  
    BG_IC_LABOR_COSTS_IND_MST_ID, 
    BG_IC_VISIT_COSTS_IND_MST_ID
]

# Drug and supply costs can be done either by drug and supply or at the intervention level 
# (only one at a time, otherwise we'd be double-counting).
BG_IC_DRUG_SUPPLY_COSTS_IND_SET = [
    BG_IC_DRUG_SUPPLY_IND_MST_ID, 
    BG_IC_INTERV_IND_MST_ID
]

# Indicators that should only show up in disease-specific mode in IHT or for TB app.
BG_IC_DIS_SPEC_IND_SET = [
    BG_IC_LABOR_COSTS_IND_MST_ID, 
    BG_IC_VISIT_COSTS_IND_MST_ID
]

# Indicators that should only show up for TB app or in IHT specific mode.
BG_IC_TB_IHT_SPEC_IND_TYPE_MST_IDS = [
    BG_IC_LOG_SUPPLY_CHAIN_COSTS_MST_ID, 
    BG_IC_WASTAGE_COSTS_MST_ID
]

# Cost categories 

BG_IC_DRUGS_SUPPLIES_CC_MST_ID = 1
BG_IC_LABOR_CC_MST_ID          = 2
BG_IC_VISITS_CC_MST_ID         = 3
BG_IC_NUM_COST_CATEGORIES      = 3

# Is the user costing intervention budget indicators or drug/supply budget indicators?

BG_IC_COST_INTERVENTIONS_MST_ID  = 1
BG_IC_COST_DRUGS_SUPPLIES_MST_ID = 2

#   *************************   PC indicator types   **************************
# 
BG_PC_COST_SECT_IND_MST_ID             = 1
BG_PC_COST_SUBSECT_IND_MST_ID          = 2
BG_PC_ACTIVITY_IND_MST_ID              = 3
BG_PC_COST_SECT_PPLI_IND_MST_ID        = 4
BG_PC_COST_SUBSECT_PPLI_IND_MST_ID     = 5
BG_PC_ACTIVITY_PPLI_IND_MST_ID         = 6
BG_PC_COST_SECT_SOC_ENAB_IND_MST_ID    = 7
BG_PC_COST_SUBSECT_SOC_ENAB_IND_MST_ID = 8
BG_PC_ACTIVITY_SOC_ENAB_IND_MST_ID     = 9
BG_PC_NUM_IND_TYPES                    = 9

# Standard indicators
BG_PC_STD_IND_TYPE_MST_IDS = [BG_PC_COST_SECT_IND_MST_ID, BG_PC_COST_SUBSECT_IND_MST_ID, BG_PC_ACTIVITY_IND_MST_ID]

# PPLI indicators
BG_PC_PPLI_IND_TYPE_MST_IDS = [BG_PC_COST_SECT_PPLI_IND_MST_ID, BG_PC_COST_SUBSECT_PPLI_IND_MST_ID, BG_PC_ACTIVITY_PPLI_IND_MST_ID]

# Social enablers indicators
BG_PC_SOC_ENAB_IND_TYPE_MST_IDS = [BG_PC_COST_SECT_SOC_ENAB_IND_MST_ID, BG_PC_COST_SUBSECT_SOC_ENAB_IND_MST_ID, BG_PC_ACTIVITY_SOC_ENAB_IND_MST_ID]

# All possible PC indicators

BG_PC_IND_TYPE_MST_IDS = [
    BG_PC_COST_SECT_IND_MST_ID, BG_PC_COST_SUBSECT_IND_MST_ID, BG_PC_ACTIVITY_IND_MST_ID,
    BG_PC_COST_SECT_PPLI_IND_MST_ID, BG_PC_COST_SUBSECT_PPLI_IND_MST_ID, BG_PC_ACTIVITY_PPLI_IND_MST_ID,
    BG_PC_COST_SECT_SOC_ENAB_IND_MST_ID, BG_PC_COST_SUBSECT_SOC_ENAB_IND_MST_ID, BG_PC_ACTIVITY_SOC_ENAB_IND_MST_ID
]

BG_PC_COST_SECT_IND_TYPE_MST_IDS = [BG_PC_COST_SECT_IND_MST_ID, BG_PC_COST_SECT_PPLI_IND_MST_ID, BG_PC_COST_SECT_SOC_ENAB_IND_MST_ID]

BG_PC_COST_SUBSECT_IND_TYPE_MST_IDS = [BG_PC_COST_SUBSECT_IND_MST_ID, BG_PC_COST_SUBSECT_PPLI_IND_MST_ID, BG_PC_COST_SUBSECT_SOC_ENAB_IND_MST_ID]

BG_PC_ACTIVITY_IND_TYPE_MST_IDS = [BG_PC_ACTIVITY_IND_MST_ID, BG_PC_ACTIVITY_PPLI_IND_MST_ID, BG_PC_ACTIVITY_SOC_ENAB_IND_MST_ID]

#   *************************   IS indicator types   **************************

# Note that Total facility operating costs used to be a group indicator that aggregated facility water, electric, non utility operating costs.  However, we no longer have
# that breakdown available, so now it because a regular lump sum indicator by facility type.

# BG_IS_CONSTR_LAND_IND_MST_ID             = 1
BG_IS_CONSTR_BUILD_IND_MST_ID            = 2
BG_IS_REHAB_COSTS_IND_MST_ID             = 3
BG_IS_MED_EQUIPMENT_IND_MST_ID           = 4 # Used by TB Costing. Also used by IHT
BG_IS_FURNITURE_IND_MST_ID               = 5
BG_IS_VEHICLES_IND_MST_ID                = 6
BG_IS_COMMUNIC_EQUIP_IND_MST_ID          = 7
#BG_IS_FACIL_NON_UTIL_OP_COSTS_IND_MST_ID = 8
# BG_IS_FACIL_ELECTR_COSTS_IND_MST_ID      = 9
# BG_IS_FACIL_WATER_COSTS_IND_MST_ID       = 10
BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID      = 11
BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID      = 12
BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID        = 13
BG_IS_ICT_MAINT_REPAIR_IND_MST_ID        = 14
BG_IS_TOTAL_NEW_FACIL_COSTS_IND_MST_ID   = 15
BG_IS_TOTAL_OP_COSTS_IND_MST_ID          = 16
# BG_IS_TOTAL_CONSTR_COSTS_IND_MST_ID      = 17
BG_IS_TOTAL_FACIL_OP_COSTS_IND_MST_ID    = 18
BG_IS_TOTAL_VEHIC_OP_COSTS_IND_MST_ID    = 19
# BG_IS_ProgMgmtInd            = 20
# BG_IS_ProgMgmtOtherActInd    = 21
BG_IS_MED_EQUIP_MAINT_CALIB_COSTS_TB_IND_MST_ID = 22 # Only used by TB Costing
BG_IS_PROG_MGMT_COST_SECT_IND_MST_ID    = 23
BG_IS_PROG_MGMT_COST_SUBSECT_IND_MST_ID = 24
BG_IS_PROG_MGMT_ACTIVITY_IND_MST_ID     = 25

BG_IS_NUM_IND_TYPES            = 17

# IS has no need of a current and master ID for the other non-facility-based indicator entry but BG does. 
BG_IS_OTHER_NON_FACIL_BASED_MST_ID = 'Non-facility-based-ID'

# All possible IS indicators

BG_IS_IND_TYPE_MST_IDS = [
    # BG_IS_CONSTR_LAND_IND_MST_ID,
    BG_IS_CONSTR_BUILD_IND_MST_ID,
    BG_IS_REHAB_COSTS_IND_MST_ID,
    BG_IS_MED_EQUIPMENT_IND_MST_ID,
    BG_IS_FURNITURE_IND_MST_ID,
    BG_IS_VEHICLES_IND_MST_ID,
    BG_IS_COMMUNIC_EQUIP_IND_MST_ID,
    #BG_IS_FACIL_NON_UTIL_OP_COSTS_IND_MST_ID,
    # BG_IS_FACIL_ELECTR_COSTS_IND_MST_ID,
    # BG_IS_FACIL_WATER_COSTS_IND_MST_ID,
    BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID,
    BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID,
    BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID,
    BG_IS_ICT_MAINT_REPAIR_IND_MST_ID,
    BG_IS_TOTAL_NEW_FACIL_COSTS_IND_MST_ID,
    BG_IS_TOTAL_OP_COSTS_IND_MST_ID,
    # BG_IS_TOTAL_CONSTR_COSTS_IND_MST_ID,
    BG_IS_TOTAL_FACIL_OP_COSTS_IND_MST_ID,
    BG_IS_TOTAL_VEHIC_OP_COSTS_IND_MST_ID,
    BG_IS_PROG_MGMT_COST_SECT_IND_MST_ID,
    BG_IS_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
    BG_IS_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

# IS indicators that appear for TB app.
BG_IS_TB_IHT_SPEC_IND_TYPE_MST_IDS = [
    BG_IS_MED_EQUIPMENT_IND_MST_ID,
    BG_IS_MED_EQUIP_MAINT_CALIB_COSTS_TB_IND_MST_ID,
]

BG_IS_TB_ONLY_IND_TYPE_MST_IDS = [
  BG_IS_MED_EQUIP_MAINT_CALIB_COSTS_TB_IND_MST_ID,  
]

# All IS indicators than can be displayed by a dynamic facility type

BG_IS_FACILITY_TYPE_IND_SET = [
    # BG_IS_CONSTR_LAND_IND_MST_ID, 
    BG_IS_CONSTR_BUILD_IND_MST_ID,
    BG_IS_REHAB_COSTS_IND_MST_ID, 
    BG_IS_MED_EQUIPMENT_IND_MST_ID, 
    BG_IS_FURNITURE_IND_MST_ID,
    BG_IS_VEHICLES_IND_MST_ID, 
    BG_IS_COMMUNIC_EQUIP_IND_MST_ID, 
    #BG_IS_FACIL_NON_UTIL_OP_COSTS_IND_MST_ID,
    # BG_IS_FACIL_ELECTR_COSTS_IND_MST_ID, 
    # BG_IS_FACIL_WATER_COSTS_IND_MST_ID,
    BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID, 
    BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID,
    BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID, 
    BG_IS_ICT_MAINT_REPAIR_IND_MST_ID,
    BG_IS_TOTAL_NEW_FACIL_COSTS_IND_MST_ID, 
    BG_IS_TOTAL_OP_COSTS_IND_MST_ID, 
    # BG_IS_TOTAL_CONSTR_COSTS_IND_MST_ID,
    BG_IS_TOTAL_FACIL_OP_COSTS_IND_MST_ID, 
    BG_IS_TOTAL_VEHIC_OP_COSTS_IND_MST_ID,
    BG_IS_MED_EQUIP_MAINT_CALIB_COSTS_TB_IND_MST_ID,
]

# All IS indicators that can be displayed by a single Non-facility based indicator in addition to the facility type indicators. The last three are in here because one or more 
# constituent indicators are also have a single Non-facility based line that needs to be accounted for. The first six are numbered here:
#
#     New facilities
#         1. Vehicles
#         2. Communication equipment
#
#     Operating costs
#         Total vehicle operating costs
#             3. Vehicle fuel expenses
#             4. Vehicle maintenace and repair
#             5. Driver salary
#         6. ICT maintenance and repair
#        
#        
#  Since 1 and 2 are part of the "New facilities" aggregate indicator type, if aggregating "New facilities", there needs to be a Non-facility based indicator line.
#  Since 3-5 are part of "Total vehicle operating costs" aggregate indicator type, which is in turn part of "Operating costs" aggregate indicator type, if 
#  aggregating "Operating costs" or "Total vehicle operating costs", there needs to be a Non-facility based indicator line. 
BG_IS_NON_FACIL_IND_SET = [
    BG_IS_VEHICLES_IND_MST_ID, 
    BG_IS_COMMUNIC_EQUIP_IND_MST_ID,
    BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID,
    BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID, 
    BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID,
    BG_IS_ICT_MAINT_REPAIR_IND_MST_ID, 
    BG_IS_TOTAL_NEW_FACIL_COSTS_IND_MST_ID, 
    BG_IS_TOTAL_VEHIC_OP_COSTS_IND_MST_ID, 
    BG_IS_TOTAL_OP_COSTS_IND_MST_ID
]

# This set contains all IS indicators that can be aggregated and disaggregated by facility type, NOT including indicators that use Non-facility-based
BG_IS_FACIL_AGGR_IND_SET = [
    # BG_IS_CONSTR_LAND_IND_MST_ID, 
    BG_IS_CONSTR_BUILD_IND_MST_ID,
    BG_IS_REHAB_COSTS_IND_MST_ID, 
    BG_IS_MED_EQUIPMENT_IND_MST_ID, 
    BG_IS_FURNITURE_IND_MST_ID,
    #BG_IS_FACIL_NON_UTIL_OP_COSTS_IND_MST_ID,
    # BG_IS_FACIL_ELECTR_COSTS_IND_MST_ID, 
    # BG_IS_FACIL_WATER_COSTS_IND_MST_ID,
    BG_IS_TOTAL_FACIL_OP_COSTS_IND_MST_ID
]

# This set contains all IS indicators that can be aggregated and disaggregated by facility type PLUS an additional slot for non-facility-based related costs.
BG_IS_FACIL_NON_FACIL_AGGR_IND_SET = [
    BG_IS_VEHICLES_IND_MST_ID, 
    BG_IS_COMMUNIC_EQUIP_IND_MST_ID,
    BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID,
    BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID, 
    BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID,
    BG_IS_ICT_MAINT_REPAIR_IND_MST_ID, 
]

# This set contains all IS indicators that can be aggregated and disaggregated by costing groups. 
BG_IS_GROUP_IND_SET = [
    BG_IS_TOTAL_NEW_FACIL_COSTS_IND_MST_ID, 
    BG_IS_TOTAL_OP_COSTS_IND_MST_ID,
    # BG_IS_TOTAL_CONSTR_COSTS_IND_MST_ID, 
    # BG_IS_TOTAL_FACIL_OP_COSTS_IND_MST_ID, now a lump sum indicator
    BG_IS_TOTAL_VEHIC_OP_COSTS_IND_MST_ID
]

# This set contains all IS indicators that fall under the total new facilities group.
BG_IS_NEW_FACIL_IND_SET = [
    # BG_IS_CONSTR_LAND_IND_MST_ID, 
    BG_IS_CONSTR_BUILD_IND_MST_ID,
    BG_IS_REHAB_COSTS_IND_MST_ID, 
    BG_IS_MED_EQUIPMENT_IND_MST_ID, 
    BG_IS_FURNITURE_IND_MST_ID,
    BG_IS_VEHICLES_IND_MST_ID, 
    BG_IS_COMMUNIC_EQUIP_IND_MST_ID, 
    # BG_IS_TOTAL_CONSTR_COSTS_IND_MST_ID
]

# This set contains all IS groups that fall under the total construction costs group.
BG_IS_CONSTR_COSTS_IND_SET = [
    # BG_IS_CONSTR_LAND_IND_MST_ID, 
    BG_IS_CONSTR_BUILD_IND_MST_ID
]

# This set contains all IS groups that fall under the total operating costs group.
BG_IS_OP_COSTS_IND_SET = [
    #BG_IS_FACIL_NON_UTIL_OP_COSTS_IND_MST_ID,
    # BG_IS_FACIL_ELECTR_COSTS_IND_MST_ID, 
    # BG_IS_FACIL_WATER_COSTS_IND_MST_ID,
    BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID, 
    BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID,
    BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID, 
    BG_IS_ICT_MAINT_REPAIR_IND_MST_ID, 
    BG_IS_TOTAL_FACIL_OP_COSTS_IND_MST_ID,
    BG_IS_TOTAL_VEHIC_OP_COSTS_IND_MST_ID
]

# This set contains all IS groups that fall under the total facility operating costs group.
# BG_IS_FACIL_OP_COSTS_IND_SET = [
#     BG_IS_FACIL_NON_UTIL_OP_COSTS_IND_MST_ID,
#     BG_IS_FACIL_ELECTR_COSTS_IND_MST_ID, 
#     BG_IS_FACIL_WATER_COSTS_IND_MST_ID
# ]

# This set contains all IS groups that fall under the total vehicle operating costs group.  
BG_IS_VEHIC_OP_COSTS_IND_SET = [
    BG_IS_VEHIC_FUEL_EXPENSE_IND_MST_ID,
    BG_IS_VEHIC_MAINT_REPAIR_IND_MST_ID, 
    BG_IS_VEHIC_DRIVER_SAL_IND_MST_ID
]

# Programme management
BG_IS_PROG_MGMT_IND_TYPE_MST_IDS = [
    BG_IS_PROG_MGMT_COST_SECT_IND_MST_ID, 
    BG_IS_PROG_MGMT_COST_SUBSECT_IND_MST_ID, 
    BG_IS_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

#  *************************   HW indicator types  ***************************

# Notes: BG_HW_INSERV_TRAIN_SPEC_TRAIN_DET_IND_MST_ID and BG_HW_INSERV_TRAIN_PERC_SAL_MST_ID, BG_HW_INSERV_TRAIN_PERC_SAL_SINGLE_STAFF_TYPE_MST_ID are mutually exclusive. The 
# former one should only be shown/costed if the user is using the specific training details approach in HW and the latter two should only be shown if the user is using the 
# percent of salary approach.

BG_HW_TOTAL_COMP_MST_ID                              = 1 # by staffing level, compensation type, dynamic staff type (aggregable by staff type)
BG_HW_PRESERV_TRAIN_MST_ID                           = 2 # by dynamic staff type (aggregable by staff type)
BG_HW_INSERV_TRAIN_PERC_SAL_MST_ID                   = 3 # only when HW uses % salary approach; by staffing level, dynamic staff type (aggregable by staff type)
# BG_HW_PROG_MGMT_MST_ID                             = 4
BG_HW_TOTAL_COMP_SINGLE_STAFF_TYPE_MST_ID            = 5 # by dynamic staff type (not aggregable by staff type)
BG_HW_INSERV_TRAIN_SPEC_TRAIN_DET_IND_MST_ID         = 6 # only when HW uses specific training details approach; by dynamic staff type (aggregable by staff type)
BG_HW_INSERV_TRAIN_PERC_SAL_SINGLE_STAFF_TYPE_MST_ID = 7 # only when HW uses % salary approach; by dynamic staff type (not aggregable by staff type; same as BG_HW_TOTAL_COMP_SINGLE_STAFF_TYPE_MST_ID)
# BG_HW_PROG_MGMT_OTHER_ACT_MST_ID                   = 8
BG_HW_PROG_MGMT_COST_SECT_IND_MST_ID                 = 9
BG_HW_PROG_MGMT_COST_SUBSECT_IND_MST_ID              = 10
BG_HW_PROG_MGMT_ACTIVITY_IND_MST_ID                  = 11

BG_HW_NUM_IND_TYPES                                  = 9

# All possible HW indicators

BG_HW_IND_TYPE_MST_IDS = [
    BG_HW_TOTAL_COMP_MST_ID, 
    BG_HW_PRESERV_TRAIN_MST_ID, 
    BG_HW_INSERV_TRAIN_PERC_SAL_MST_ID, 
    BG_HW_TOTAL_COMP_SINGLE_STAFF_TYPE_MST_ID, 
    BG_HW_INSERV_TRAIN_SPEC_TRAIN_DET_IND_MST_ID, 
    BG_HW_INSERV_TRAIN_PERC_SAL_SINGLE_STAFF_TYPE_MST_ID,
    BG_HW_PROG_MGMT_COST_SECT_IND_MST_ID,
    BG_HW_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
    BG_HW_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

#   All HW indicators that can (dis)aggregated by staff type. 
BG_HW_STAFF_AGGR_IND_SET = [
    BG_HW_TOTAL_COMP_MST_ID, BG_HW_PRESERV_TRAIN_MST_ID,
    BG_HW_INSERV_TRAIN_PERC_SAL_MST_ID, BG_HW_INSERV_TRAIN_SPEC_TRAIN_DET_IND_MST_ID
]

#   All HW indicators that that can be displayed by dynamic staff type.
BG_HW_STAFF_TYPE_IND_SET = [
    BG_HW_TOTAL_COMP_MST_ID, BG_HW_PRESERV_TRAIN_MST_ID,
    BG_HW_INSERV_TRAIN_PERC_SAL_MST_ID, BG_HW_TOTAL_COMP_SINGLE_STAFF_TYPE_MST_ID,
    BG_HW_INSERV_TRAIN_SPEC_TRAIN_DET_IND_MST_ID, BG_HW_INSERV_TRAIN_PERC_SAL_SINGLE_STAFF_TYPE_MST_ID
]

# Programme management
BG_HW_PROG_MGMT_IND_TYPE_MST_IDS = [
    BG_HW_PROG_MGMT_COST_SECT_IND_MST_ID, 
    BG_HW_PROG_MGMT_COST_SUBSECT_IND_MST_ID, 
    BG_HW_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

#   *************************   HS indicator types  ***************************

BG_HS_COSTING_COST_SECT_IND_MST_ID      = 5
BG_HS_COSTING_COST_SUBSECT_IND_MST_ID   = 6
BG_HS_COSTING_ACTIVITY_IND_MST_ID       = 7
BG_HS_PROG_MGMT_COST_SECT_IND_MST_ID    = 8
BG_HS_PROG_MGMT_COST_SUBSECT_IND_MST_ID = 9
BG_HS_PROG_MGMT_ACTIVITY_IND_MST_ID     = 10
BG_HS_NUM_IND_TYPES                     = 6

# All possible HS indicators

BG_HS_IND_TYPE_MST_IDS = [
    BG_HS_COSTING_COST_SECT_IND_MST_ID, 
    BG_HS_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_HS_COSTING_ACTIVITY_IND_MST_ID,
    BG_HS_PROG_MGMT_COST_SECT_IND_MST_ID,
    BG_HS_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
    BG_HS_PROG_MGMT_ACTIVITY_IND_MST_ID
]

# HS costing indicators

BG_HS_COSTING_IND_TYPE_MST_IDS = [
    BG_HS_COSTING_COST_SECT_IND_MST_ID, 
    BG_HS_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_HS_COSTING_ACTIVITY_IND_MST_ID,
]

# HS programme management indicators

BG_HS_PROG_MGMT_IND_TYPE_MST_IDS = [
    BG_HS_PROG_MGMT_COST_SECT_IND_MST_ID, 
    BG_HS_PROG_MGMT_COST_SUBSECT_IND_MST_ID, 
    BG_HS_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

BG_HS_SECTION_IND_MST_IDS = [
    BG_HS_COSTING_COST_SECT_IND_MST_ID, 
    BG_HS_PROG_MGMT_COST_SECT_IND_MST_ID, 
]

BG_HS_SUBSECTION_IND_MST_IDS = [
    BG_HS_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_HS_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
]

BG_HS_ACT_IND_MST_IDS = [
    BG_HS_COSTING_ACTIVITY_IND_MST_ID,
    BG_HS_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

# HIS or programme management?
BG_HS_COSTING_IND_TYPE_DIVISION_MST_ID   = 1
BG_HS_PROG_MGMT_IND_TYPE_DIVISION_MST_ID = 2

#   *************************   GV indicator types  ***************************

BG_GV_COSTING_COST_SECT_IND_MST_ID      = 5
BG_GV_COSTING_COST_SUBSECT_IND_MST_ID   = 6
BG_GV_COSTING_ACTIVITY_IND_MST_ID       = 7
BG_GV_PROG_MGMT_COST_SECT_IND_MST_ID    = 8
BG_GV_PROG_MGMT_COST_SUBSECT_IND_MST_ID = 9
BG_GV_PROG_MGMT_ACTIVITY_IND_MST_ID     = 10
BG_GV_NUM_IND_TYPES                     = 6

# All possible GV indicators

BG_GV_IND_TYPE_MST_IDS = [
    BG_GV_COSTING_COST_SECT_IND_MST_ID, 
    BG_GV_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_GV_COSTING_ACTIVITY_IND_MST_ID,
    BG_GV_PROG_MGMT_COST_SECT_IND_MST_ID,
    BG_GV_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
    BG_GV_PROG_MGMT_ACTIVITY_IND_MST_ID
]

# GV costing indicators

BG_GV_COSTING_IND_TYPE_MST_IDS = [
    BG_GV_COSTING_COST_SECT_IND_MST_ID, 
    BG_GV_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_GV_COSTING_ACTIVITY_IND_MST_ID,
]

# GV programme management indicators

BG_GV_PROG_MGMT_IND_TYPE_MST_IDS = [
    BG_GV_PROG_MGMT_COST_SECT_IND_MST_ID, 
    BG_GV_PROG_MGMT_COST_SUBSECT_IND_MST_ID, 
    BG_GV_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

BG_GV_SECTION_IND_MST_IDS = [
    BG_GV_COSTING_COST_SECT_IND_MST_ID, 
    BG_GV_PROG_MGMT_COST_SECT_IND_MST_ID, 
]

BG_GV_SUBSECTION_IND_MST_IDS = [
    BG_GV_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_GV_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
]

BG_GV_ACT_IND_MST_IDS = [
    BG_GV_COSTING_ACTIVITY_IND_MST_ID,
    BG_GV_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

# GV or programme management?
BG_GV_COSTING_IND_TYPE_DIVISION_MST_ID   = 1
BG_GV_PROG_MGMT_IND_TYPE_DIVISION_MST_ID = 2

#   *************************   HF indicator types  ***************************

BG_HF_COSTING_COST_SECT_IND_MST_ID      = 5
BG_HF_COSTING_COST_SUBSECT_IND_MST_ID   = 6
BG_HF_COSTING_ACTIVITY_IND_MST_ID       = 7
BG_HF_PROG_MGMT_COST_SECT_IND_MST_ID    = 8
BG_HF_PROG_MGMT_COST_SUBSECT_IND_MST_ID = 9
BG_HF_PROG_MGMT_ACTIVITY_IND_MST_ID     = 10
BG_HF_NUM_IND_TYPES                     = 6

# All possible HF indicators

BG_HF_IND_TYPE_MST_IDS = [
    BG_HF_COSTING_COST_SECT_IND_MST_ID, 
    BG_HF_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_HF_COSTING_ACTIVITY_IND_MST_ID,
    BG_HF_PROG_MGMT_COST_SECT_IND_MST_ID,
    BG_HF_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
    BG_HF_PROG_MGMT_ACTIVITY_IND_MST_ID
]

# HF costing indicators

BG_HF_COSTING_IND_TYPE_MST_IDS = [
    BG_HF_COSTING_COST_SECT_IND_MST_ID, 
    BG_HF_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_HF_COSTING_ACTIVITY_IND_MST_ID,
]

# HF programme management indicators

BG_HF_PROG_MGMT_IND_TYPE_MST_IDS = [
    BG_HF_PROG_MGMT_COST_SECT_IND_MST_ID, 
    BG_HF_PROG_MGMT_COST_SUBSECT_IND_MST_ID, 
    BG_HF_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

BG_HF_SECTION_IND_MST_IDS = [
    BG_HF_COSTING_COST_SECT_IND_MST_ID, 
    BG_HF_PROG_MGMT_COST_SECT_IND_MST_ID, 
]

BG_HF_SUBSECTION_IND_MST_IDS = [
    BG_HF_COSTING_COST_SUBSECT_IND_MST_ID, 
    BG_HF_PROG_MGMT_COST_SUBSECT_IND_MST_ID,
]

BG_HF_ACT_IND_MST_IDS = [
    BG_HF_COSTING_ACTIVITY_IND_MST_ID,
    BG_HF_PROG_MGMT_ACTIVITY_IND_MST_ID,
]

# HF or programme management?
BG_HF_COSTING_IND_TYPE_DIVISION_MST_ID   = 1
BG_HF_PROG_MGMT_IND_TYPE_DIVISION_MST_ID = 2

# Costing level; now used for all health systems with programme management. Note that for HS, the user cannot mix 
# HS Costing and HS Programme Management levels in the editor. Both must be costing, subsection, activity, etc.
# Used for HS, IS, HW, LG. Also note that each module needs to be able to save a different level at the same time.
# That is, HS could be at the section level, IS could be at the subsection level, etc.

BG_COST_COSTING_SECTIONS_LEVEL_MST_ID     = 1
BG_COST_COSTING_SUBSECTIONS_LEVEL_MST_ID  = 2
BG_COST_ACTIVITIES_LEVEL_MST_ID           = 3



# Global Constants }
#$I FPTagsRW.INC}
FP_Table           = 101
#  FP_Pie             = 102

# Read Write determines size of numbers in file 
FP_FileNumSpace = 20
FP_FileRDec     = 6
FP_Version      = 4.5
# Error Messages 
FP_NOERROR           = 0
FP_ERROR             = 1

# Special family planning constants }
FP_DataFileTitle     = 'Family Planning Module Data File - The POLICY Project'

FP_DatFVersion       = 7.00
FP_DatFVersion70     = 7.00   #reduced FileName.FP file size
FP_DatFVersion60     = 6.00         #added SDM method
FP_DatFVersion50     = 5.00
FP_DatFVersion43     = 4.30        #Added LAM Method}
FP_DatFVersion42     = 4.20
FP_DatFVersion41     = 4.10
FP_DatFVersion4      = 4.00

FP_All = 0
FP_MAR = 1
FP_MARUMSA = 2
FP_AllUninterpolated = 3
FP_MARUninterpolated = 4
FP_MARUMSAUninterpolated = 5

FP_DB_SheetName = {
    FP_All: 'All Data',
    FP_MAR: 'MAR',
    FP_MARUMSA: 'MARUMSA',
    FP_AllUninterpolated: 'All Data (Uninterpolated)',
    FP_MARUninterpolated: 'MAR (Uninterpolated)',
    FP_MARUMSAUninterpolated: 'MARUMSA (Uninterpolated)'
}

FP_InterpDir = 'countries/interpolated' 
FP_Uninterp  = 'countries/uninterpolated'

FP_DB_Dir = {
    FP_All : FP_InterpDir,
    FP_MAR : FP_InterpDir,
    FP_MARUMSA : FP_InterpDir,
    FP_AllUninterpolated : FP_Uninterp,
    FP_MARUninterpolated : FP_Uninterp,
    FP_MARUMSAUninterpolated : FP_Uninterp
}

FP_All_SubDir = 'All'
FP_MAR_SubDir = 'MAR'
FP_MARUMSA_SubDir = 'MARUMSA'

FP_DB_SubDir = {
    FP_All : FP_All_SubDir,
    FP_MAR : FP_MAR_SubDir,
    FP_MARUMSA : FP_MARUMSA_SubDir,
    FP_AllUninterpolated : FP_All_SubDir,
    FP_MARUninterpolated : FP_MAR_SubDir,
    FP_MARUMSAUninterpolated : FP_MARUMSA_SubDir
}

FP_DB_Version = {
    FP_All : 'V2',
    FP_MAR : 'V2',
    FP_MARUMSA : 'V2',
    FP_AllUninterpolated : 'V2',
    FP_MARUninterpolated : 'V2',
    FP_MARUMSAUninterpolated : 'V2'
}

FP_ShowNewFeatures = True

# Age constants }
FP_All_Ages = 0
FP_A15_19   = 1
FP_A20_24   = 2
FP_A25_29   = 3
FP_A30_34   = 4
FP_A35_39   = 5
FP_A40_44   = 6
FP_A45_49   = 7
FP_MAX_AGE  = 7

#   FP_AgeCats : array[FP_All_Ages..FP_MAX_AGE] of string =
#                  ('All Ages', '15-19', '20-24', '25-29',
#                   '30-34', '35-39', '40-44',
#                   '45-49')

# Need constants }
FP_all_need     = 0
FP_spacing      = 1
FP_limiting     = 2
FP_max_need     = 2


# Time constants
FP_FirstDBYear   = 1990
FP_FinalDBYear   = 2030

# Age group constants }
FP_Single_Age_Group     = 1
FP_Five_Year_Age_Groups = 2

# Goal constants }
FP_Unmet_need     = 1
FP_Desired_TFR    = 2
FP_Prevalence     = 3
FP_TFR            = 4  #Total fertility rate goal
FP_mCPR           = 5  #modern contraceptive prevalence goal
FP_AddMdrnMeth    = 6  #additional modern method users goal
FP_Pregnancy      = 7

FP_TFR_MT_TFRGOAL   = 0
FP_ASFR_TFRGOAL     = 1

# Abortion constants }
FP_specify_TAR         = 1
FP_calculate_abortions = 2


# Method constants
FP_all_methods    = 0
# condoms
FP_CondomFM       = 1
FP_CondomML       = 2
# sterilization
FP_fster          = 3
FP_mster          = 4

# vaginal barriers and tablets
FP_vag_barrier    = 5
FP_VFT            = 6

# traditional contraceptive methods
FP_withdrawal     = 7
FP_PerAbst        = 8 # periodic abstinence
FP_Traditional    = 9

# injectables
FP_inj3month      = 10
FP_inj2month      = 11
FP_inj1month      = 12
FP_inj6month      = 13
FP_Uninject       = 14

# implants
FP_Implanon       = 15
FP_Jadelle        = 16
FP_Sino_Implant   = 17

# pills
FP_StandardPill   = 18
FP_Progestin      = 19
FP_Peri_coital    = 20

# IUD
FP_IUDCopper      = 21
FP_IUDLngIus      = 22

FP_LAM            = 23
FP_SDM            = 24
FP_other          = 25

FP_LastDefMethod  = 25
FP_MstDefMeth     = 31
FP_CustMethodMax  = 12
FP_Max_Methods    = FP_MstDefMeth+FP_CustMethodMax
FP_first_method   = 1
FP_last_method    = FP_Max_Methods

FP_Condoms        = [FP_CondomML, FP_CondomFM]
FP_Sterilizations = [FP_fster,FP_mster]
FP_Injectables    = [FP_inj3month,FP_inj2month,FP_inj1month,
                    FP_inj6month, FP_Uninject]
FP_Implants       = [FP_Implanon,FP_Jadelle,FP_Sino_Implant]
FP_IUD            = [FP_IUDCopper, FP_IUDLngIus]
FP_Pills          = [FP_StandardPill, FP_Progestin, FP_Peri_coital]
FP_FertAware      = [FP_LAM,FP_SDM]
FP_Barrier        = [FP_vag_barrier,FP_VFT]


FP_TRAD_METH  = [FP_PerAbst, FP_withdrawal, FP_Traditional]
# FP_AllMethods = [1..FP_Max_Methods]

# # FP_otherMethods = FP_AllMethods-(FP_Condoms  + FP_Sterilizations + FP_Injectables
#                                 + FP_Implants + FP_Pills + FP_IUD+FP_TRAD_METH
#                                 + FP_FertAware+ FP_Barrier)

# # default Type of Methods(shortTerm/LongTerm)
# FP_DEFPermMeth      = [FP_fster,FP_mster]
# FP_DefLongTermMeth  = [FP_Implanon, FP_Jadelle, FP_Sino_Implant, FP_IUDCopper,
#                         FP_IUDLngIus]
# FP_DefShortTermMeth = [FP_condomFM, FP_condomML, FP_VFT, FP_Lam, FP_Other,
#                         FP_Uninject, FP_SDM, FP_vag_barrier]+FP_Injectables+FP_Pills



FP_PermanentMethods  = [FP_fster,FP_mster]
FP_LongTermtMethods  = [FP_Implanon, FP_Jadelle, FP_Sino_Implant, FP_IUDCopper, 
                         FP_IUDLngIus]
FP_ShortTermMethods = ([FP_CondomFM, FP_CondomML, FP_VFT, FP_LAM, FP_other, 
                         FP_Uninject, FP_SDM, FP_vag_barrier] + FP_Injectables + FP_Pills)

# FP_ActiveMethodsDefault = [FP_StandardPill,FP_IUDCopper,FP_inj3month,FP_condomML,
#                             FP_fster,FP_mster,FP_Jadelle,FP_LAM,FP_vag_barrier,
#                             FP_VFT,FP_withdrawal]

# #      Condom, sterilization, Injectable, Implant, Pill, IUD, LAM, SDM, vaginal barrier, spermicide
# # FP_ModernCPRMeth = FP_Condoms+FP_Sterilizations+FP_Injectables+FP_Implants
# #                 + FP_IUD+FP_Pills+[FP_LAM, FP_SDM, FP_vag_barrier]


#   # Fecundity constants 
#   FP_Fecundity_constant : array[FP_All_Ages..FP_MAX_AGE] of single =
#                           (1.08, 1.02, 1.02, 1.03, 1.04, 1.12, 1.33, 1.08)

  # Source constants 
FP_all_sources        = 0
FP_max_sources        = 10
FP_PUB_SRC            = 1

# Scale constants 
FP_units             = 1
FP_thousands         = 2
FP_millions          = 3
FP_thousand_millions = 4

# Pregnancy constants 
FP_all_PREG = 0
FP_wanted   = 1
FP_unwanted = 2
FP_mistimed = 3
FP_MAX_PREG = 3

# Method Mix Editor style constants 
FP_METHMIXED_1 = 1
FP_METHMIXED_2 = 2
FP_METHMIXED_3 = 3
FP_METHMIXED_4 = 4

FP_LAM0_1   = 1
FP_LAM2_3   = 2
FP_LAM4_5   = 3

# Indicator constants
FP_SecFertility        = 1
FP_SecImpactFP         = 2
FP_SecWRA              = 2 #bump back up after removing FP_ShowNewFeatures
FP_Sec_DemoEvents      = 3 #formerly FP_SecPregnancies
FP_SecRelativeRisk     = 4
FP_SecMortRate         = 5
FP_PACSec              = 6
FP_SecGrossCost        = 7
FP_SecSummary          = 8
FP_MAXSEC              = 8

FP_FirstInd            = 201
FP_cmAvgEff            = 201
FP_cmPrevalence        = 202
FP_cmUnmetNeed         = 203
FP_cmTFR               = 204
FP_cmWRA               = 205
FP_cmMWRA              = 206
FP_cmUsers             = 207
FP_cmAcceptors         = 208
FP_cmCYP               = 209
FP_cmGrossCost         = 210
FP_cmCostPerUser       = 211
FP_cmRevenue           = 212
FP_cmNetCost           = 213
FP_cmCommodities       = 214
FP_cmBirths            = 215
FP_cmAbortions         = 216
FP_cmPregnancies       = 217

FP_cmTF                = 218
FP_cmGrowthRates       = 219
FP_cmFPSumInp          = 220
FP_cmFPSumOut          = 221

# PAC Indicator Constants 
FP_cmMaternDeathTotal  = 222
FP_cmMDWantBirth       = 223
FP_cmMDUnWantBirth     = 224
FP_cmMDTreatIllAbort   = 225
FP_cmMDUnTreatIllAbort = 226
FP_cmTotDeathAvert     = 227
FP_cmPACAbortions      = 228
FP_cmIllAbortions      = 229
FP_cmIllAbortNoTreat   = 230
FP_cmIllAbortTreat     = 231
FP_cmTreatIllAbort     = 232
FP_cmUntreatIllAbort   = 233
FP_cmMMR               = 234
FP_cmMaternDeathAbort  = 235
FP_cmPACSummary        = 236

# Child Survival 
FP_cmPerBirthRisk      = 237
FP_cmFertAdjIMR        = 238
FP_cmFertAdjU5MR       = 239

#relative risk indicators
FP_cmAgeAndBirth       = 240
FP_cmBirthInterv       = 241
FP_cmBirthAverted      = 242
FP_cmModernCPR         = 243
FP_cmAddModernUsers    = 244
FP_cmPregAvertModern   = 245
FP_cmMatDeathAvertMdrn = 246
FP_cmAbortAvertedMdrn  = 247
FP_cmPercWomenUnmetNeed= 248
FP_cmPercSatisfiedMdrn = 249
FP_cmPreganancyRate    = 250


FP_MAXIND              = 250#239

#Relative Risk
FP_NumRelativeRiskCat = 15
#constants for order of births
FP_RRFirstBirth      = 1
FP_RRSecToThirdBirth = 2
FP_RRMoreFourBirth   = 3
FP_RRBirthOrderMax   = 3

#mothers ages
FP_RRLess18   = 1
FP_RR18To34   = 2
FP_RRMore34   = 3
FP_RRAgeOrderMax = 3

#  FP_RRFirstBirth       = 1
FP_RRLess18Months     = 2
FP_RR18to23Months     = 3
FP_RRMore24Months     = 4
FP_RRBirthIntervalMax = 4
FP_RRExtraBirthIntervals = 5

FP_RRIntrcept = 1
FP_RRC1       = 2
FP_RRC2       = 3

FP_RRBOGreater3 = 1
FP_RRMAGreat35  = 2
FP_RRMA18To34   = 3



#Constants used for Results
FP_MethodInd        = 11
FP_SourceInd        = 12
FP_AgeInd           = 13
FP_NeedInd          = 14
FP_PREGInd          = 15
FP_SINGLE_METHODIND = 16
FP_NO_TOTAL         = 17
FP_RelRiskInd       = 18
FP_SUMInd           = 19

# PAC Input Constants 
FP_PerAbortLegal            = 1
FP_PerIllAbortTreat         = 2
FP_PerLegalAbortTreat       = 3
FP_MMRMaternDeath           = 4
FP_PerMaternDeathAbortion   = 5
FP_MortalityUntreatVsTreat  = 6
FP_AnnPostAbortCare         = 7
FP_CostAbortCompTreat       = 8
FP_CostCounServCase         = 9

FP_PACMaternalSet = [FP_MMRMaternDeath]

# PAC Output/Calculated Constants 
FP_WantedPreg               = 1
FP_UnwantedPreg             = 2
FP_PerAbortNeedTreatReceive = 3
FP_MMRBirthsOne             = 4
FP_AbortDeathsPerAbort      = 5
FP_DeathTreatIllAbort       = 6
FP_DeathUntreatIllAbort     = 7
FP_MDTotal                  = 8
FP_MDWantBirths             = 9
FP_MDUnwantBirths           = 10
FP_MDTreatIllAbort          = 11
FP_MDUntreatIllAbort        = 12
FP_TotDeathAvert            = 13
FP_Births                   = 14
FP_WantedBirths             = 15
FP_UnwantedBirths           = 16
FP_Abortions                = 17
FP_IllAbortions             = 18
FP_IllAbortNoTreat          = 19
FP_IllAbortTreat            = 20
FP_TreatIllAbort            = 21
FP_UntreatIllAbort          = 22
FP_MMR                      = 23
FP_MMRBirthsTwo             = 24
FP_MMRAbort                 = 25
FP_MaternDeathAbort         = 26
FP_LivesSaved               = 27
FP_CostTreatUnsafeAbort     = 28
FP_CostPerMaternDeathAvert  = 29

# Calculation method constants 
FP_TotFec = 1   #Use total fecundity and proximate determinants
FP_PregRate = 2   # Use pregnancy rate approach 
FP_CalculationMethods = [FP_TotFec, FP_PregRate]

FP_MiscarriagePercent       = 15.0 # 15%

# Child Survival 
FP_PerBirthRisk  = 1
FP_FertAdjIMR    = 2
FP_FertAdjU5MR   = 3
FP_RelRiskyBirth = 4
FP_RelIMR        = 5
FP_RelU5MR       = 6

# EC 
FP_AvgCoitFr = 1
FP_EffEmerContr = 2
FP_TF = 3

FP_PercUnpro = 1
FP_PercElig = 2
FP_IndexEC = 3

# Policy analysis 
FP_BYear    = 0
FP_TYear    = 1

#easy famplan file
FP_EasyFamPlanDBFileName2 = 'EasyFamPlan_Source.xlsx'

# Help file constants 
FP_Help_Configuration           = 'Configuration'
FP_Help_MethodMix               = 'Method_mix'
FP_Help_SourceMix               = 'Source_mix'
FP_Help_ProximateDeterminants   = 'Proximate_determinants'
FP_Help_ChildSurvival           = 'Child_survival'
FP_Help_CostsOfServices         = 'Costs_of_services'
FP_Help_Fees                    = 'Fees'
FP_Help_MethodAttributes        = 'Method_attributes'
FP_Help_LAM                     = 'LAM'
FP_Help_Effectiveness           = 'Effectiveness'
FP_Help_ReducingUnmetNeed       = 'Reducing_unmet_need_for_contraception'
FP_Help_TotalWantedFertility    = 'Total_wanted_fertility'
FP_Help_ReachingGoalPrevalence  = 'Reaching_a_goal_for_contraceptive_prevalence'
FP_Help_ReachingGoalTFR         = 'Reaching_a_goal_for_total_fertility_rate'
FP_Help_PostAbortionCare        = 'Post_abortion_care'
FP_Help_RelativeRisk            = 'fertility_relative_risk'

FP_EasyAWarning  = 1
FP_EasyABWarning = 2
FP_EasyBWarning  = 3
FP_EasyCWarning  = 4


# FPEdFm glow button constants 
FP_configBtn     = 1
FP_methodMixBtn  = 2
FP_GoalBtn       = 3
FP_PACBtn        = 4
FP_RelRiskBtn    = 5
FP_policyBtn     = 6
FP_ResultsBtn    = 7


# #EasyFamplan tags
# FP_countryCodeTag   = 'Code'
# #
# FP_countryCodeTag_1   = 'ISO'
# FP_countryCodeTag_2   = 'Country Code'

# FP_countryNameTag   = 'Country'
# FP_surveyYrTag      = 'Year'
# FP_SterilityTag     = 'Childless Women ages 45-49 (Sterility)'
# #
# FP_SterilityTag_1     = 'Childless Women45-49(Sterility)'
# FP_SourceTag        = 'Group'#'source'
# FP_PillStandardTag  = 'Pill (standard)'
# FP_PillProgestinTag = 'Pill (progestin only)l'
# FP_PillPCTag        = 'Pill (PCC)'
# FP_IUDCopperTag     = 'IUD (Copper T, 10 years)'
# FP_IUDLNGTag        = 'IUD (LNG-IUS 5 years)'
# FP_InjDepoTag       = 'Injection (Depo 3 months default)'
# #                       Injectable(Depo3months default)
# FP_InjDepoTag_1       = 'Injectable(Depo3months default)'
# FP_InjNoristeratTag = 'Injectable(Noristerat 2 months)'
# FP_InjLunelleTag    = 'Injectable(Lunelle 1 months)'
# FP_Inj6monthTag     = 'Injectable(6 month)'
# FP_InjUnijectTag    = 'Injectable (Uniject)'
# FP_CondomMaleTag    = 'Condom (Male)'
# FP_CondomFemTag     = 'Condom (Female)'
# FP_SterilFemTag     = 'Sterilization (Female)'
# FP_SterilMaleTag    = 'Sterilization (Male)'
# FP_ImpJadelleTag    = 'Implant (Jadelle 5 years default)'
# #
# FP_ImpJadelleTag_1    = 'Implant(Jadelle 5years Default)'
# FP_ImpImplanonTag   = 'Implant (Implanon 3 years)'
# FP_ImpSinoTag       = 'Implant (Sino-Implant 4 years)'
# FP_LAMTag           = 'Fertility Awaremeness (Amenorrhea (LAM))'
# #
# FP_LAMTag_1           = 'Fertility Awaremeness (LAM)'
# FP_SDMTag           = 'Fertility Awareness (SDM)'
# FP_DiaphramTag      = 'Vaginal Barrier (Diaphragm)'
# FP_SpermicTageTag   = 'Vaginal Barrier (Foam or Jelly, Spermicides)'
# #
# FP_SpermicTage_1      = 'VaginalBarrier(FoamSpermicides)'
# FP_TradOtherTag     = 'Traditional (Other)'
# FP_WithdrawalTag    = 'Traditional (Withdrawal)'
# FP_PeriodicTag      = 'Traditional (Periodic Abstenance)'
# #
# # FP_PeriodicTag_1      = 'Traditional(PeriodicAbstinence)'  # Rhythm
# FP_PeriodicTag_1      = 'Traditional(PeriodicAbstinence)'



# FP_OtherMethTag     = 'Other Modern'
# FP_unionPercTag     = 'In Union (Curr. Married + Living togthr)'
# #
# FP_unionPercTag_1     = 'In Union'
# FP_PPITag           = 'PPI Median Duration'
# FP_unmetNeedTag     = 'Unmet need'
# FP_abortPercTag     = 'Abortions per unwanted pregnancy'
# #                         Proportion of unintended pregnancies terminated by abortion
# FP_abortPercTag_1     = 'Proportion of unintended pregnancies terminated by abortion'
# #  FP_unmetSrcTag      = 'Unmet need data sources (see "Unmet Need" tab for codes)'
# #
# #  FP_unmetSrcTag_1      = 'Unmet need data sources'
# FP_RiskYearTag      = 'RiskYear'
# FP_MAL18FirstBirthTag  = 'MA<18 & First Birth'
# FP_MAL18BO2_3Tag    = 'MA<18 & BO2_3'
# FP_MAL18BOG3Tag     = 'MA<18 & BO>3'
# FP_MA18_34FirstBirthTag  = 'MA18_34 & First Birth'
# FP_MA18_34BO2_3Tag  = 'MA18_34 & BO2_3'
# FP_MA18_34BOG3Tag   = 'MA18_34 & BO>3'
# FP_MAG34FirstBirthTag  = 'MA>35 & First Birth'
# FP_MAG34BO2_3Tag    = 'MA>35 & BO2_3'
# FP_MAG34BOG3Tag     = 'MA>35 & BO>3'
# FP_BOFirstBirthTag  = 'FirstBirth Interval'
# FP_BOlt18Tag        = '<18Months'
# FP_BO18_24Tag       = '<24Months'
# FP_BOgt24Tag        = '24 monthers or Greater'
# FP_houseHoldSizeTag = 'Household Size'
# FP_houseHoldSrcTag  = 'Household size source'
# FP_AnyFertilRiskTag = 'Any fertility risk'
# FP_MainDataSource   = 'Main data sources and references (refer to citations tab)'
# FP_MMRTag           = 'MMR'
# #
# FP_MainDataSource_1   = 'Main data sources(citationsTab)'


# FP_UnintendedPregTag  = 'Proportion of unintended pregnancies terminated by abortion'
# FP_UnsafeAbortMMRTag  = 'Ratio Unsafe abortion MMR to MMR'
# FP_UnsafeAbortionsTag = 'Proportion of abortions that are unsafe'
# FP_CPRTag             = 'CPR'
# FP_TARTag             = 'TAR'
# #
# FP_TheRestTag         = 'The Rest'
# #
# FP_SingleYearDataTag         = 'Single Year Data'

# FP_EffectivenessTag          = 'Effectiveness'

FP_MaxEditorVal = 999999999999999 #999 trillion

# Modes 
FP_StandardMode   = 1 # normal
FP_CWFMode        = 2 # CHOICE Work Flow Scenario Generator

FP_CWFBaseYrIdx = 1
FP_CWFTargYrIdx = 2
FP_CWF_MaxCPR = 85


FP_Fecundity_constant = (1.08, 1.02, 1.02, 1.03, 1.04, 1.12, 1.33, 1.08)

FP_COUNTRY_PROXY = {
    "BYR" : "UKR", # Belarus	Has data (using Ukraine as proxy)
    "BWP" : "UKR", # Botswana*	Has data (using Ukraine as proxy)
    #Cote d'Ivoire	is this a name issues?
    "KPW" : "MMK", # Dem. People's Republic of Korea	Has data (using Myanmar as proxy)
    "DJF" : "ETB",# Djibouti	Has data (using Ethiopia as proxy)
    # Ecuador	NOT in Database at al????
    "XAF" : "GNF", # Equatorial Guinea	Has data (using Guinea as proxy)
    # Eritrea	NOT in Database at al????
    "GEL" : "AMD", # Georgia	Has data (using  Armenia as proxy)
    "QAR" : "TRY", # Qatar	Has data (using Turkey  as proxy)
    "MKD" : "MDL", # Republic of North Macedonia	Has data (using  Moldova as proxy)
    "WST" : "IDR", # Samoa	Has data (using Indonesia  as proxy)
    "SAR" : "TRY", # Saudi Arabia	Has data (using Turkey  as proxy)
    "SBD" : "IDR", # Solomon Islands	Has data (using Indonesia  as proxy)
    "SSP" : "UGX", # South Sudan	Has data (using Uganda  as proxy)
    # Sri Lanka	NOT in Database at al????
    "SYP" : "TRY", # Syrian Arab Republic	Has data (using Turkey  as proxy)
    # Thailand*	Missing data- look to see who their proxie is
    "AED" : "TRY", # United Arab Emirates	Has data (using Turkey  as proxy)
    "VUV" : "IDR"# Vanatu	Has data (using Indonesia  as proxy)

}


from ...GB.Util.GBDate import GBDate

def create_simple_modvar(tag, value):
    return {'tag':tag, 'module':'TB', 'value':value}

def init_result_modvars(modvars):
    lockdownDate = GBDate('March', 2020)
    return [
            create_simple_modvar('TB_Incidence', []),
            create_simple_modvar('TB_Notification', []),
            create_simple_modvar('TB_Mortality', []),
    ]



import numpy as np

from SpectrumCommon.Const.TB import *


def TB_RRProfileDefaults(db, notif_pct, year_len):
    rr_profile = np.zeros((TB_NewRetreatLen, TB_ResistAgeLen, TB_ResistProfileLen, year_len))
    db_RR_Profile = np.nan_to_num(np.array(db['RR_Profile']), copy=True, nan=0.0, posinf=None, neginf=None)
    db['RR_Profile']  = db_RR_Profile.tolist()
    for t in range(0, year_len):
        rr_profile[TB_NewCases, TB_Child, :, t] = db_RR_Profile[TB_NewCases].copy() #Child
        rr_profile[TB_NewCases, TB_Adult, :, t] = db_RR_Profile[TB_NewCases].copy() #Adult

        rr_profile[TB_RetreatCases, TB_Child, :, t] = db_RR_Profile[TB_RetreatCases].copy() #Child
        rr_profile[TB_RetreatCases, TB_Adult, :, t] = db_RR_Profile[TB_RetreatCases].copy() #Adult

    rp_newU15 = rr_profile[TB_NewCases][TB_Child][TB_Rif_Res]
    rp_new15p = rr_profile[TB_NewCases][TB_Adult] [TB_Rif_Res]

    rp_retU15 = rr_profile[TB_RetreatCases][TB_Child][TB_Rif_Res]
    rp_ret15p = rr_profile[TB_RetreatCases][TB_Adult] [TB_Rif_Res]

    pRet      = notif_pct[TB_PrevTreatIncludeRelapseCases]
    pNew      = 1-pRet

    rp_Rif = np.zeros((TB_ResistanceLen, TB_ResistAgeLen, year_len))
    rp_Rif[TB_MDR][TB_ResistAge0_14] = pNew*(rp_newU15) + (pRet)*(rp_retU15)    
    rp_Rif[TB_MDR][TB_ResistAge15p]  = pNew*(rp_new15p) + (pRet)*(rp_ret15p)  

    rp_Rif[TB_SEN][TB_ResistAge0_14]=(1-rp_Rif[TB_MDR][TB_ResistAge0_14] )
    rp_Rif[TB_SEN][TB_ResistAge15p]=(1-rp_Rif[TB_MDR][TB_ResistAge15p] )

    return rr_profile, rp_Rif

def TB_ResistanceCoverageDefaults(year_len):
    coverages  = np.full((TB_CoverageLen, year_len), 0.50)

    coverages[0]  = 1 # db['coverage']['DST']
    coverages[1]  = 1 # db['coverage']['DST']
    coverages[2]  = 1 # db['coverage']['DST']
    coverages[3]  = 1 # db['coverage']['DST']
    return coverages


from starlette.middleware.base import BaseHTTPMiddleware


class RouterLoggingMiddleware(BaseHTTPMiddleware):
    def __init__(self, app, logger, subsystem):
        super().__init__(app)

        self._logger = logger
        self._subsystem = subsystem

    async def dispatch(self, request, call_next):
        self._logger.info(f"{request.method} {request.url}")

        response = await call_next(request)

        self._logger.info(
            f"{request.method} {request.url} {response.status_code}",
            extra={"custom_dimensions": {"subsystem": self._subsystem}},
        )

        return response



from datetime import date
import math as math

import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Const.PJ.PJNTags as pjmvt
import SpectrumCommon.Modvars.GB.GBUtil as gbu

import SpectrumCommon.Const.BG.BGConstLink as bgcl
import SpectrumCommon.Const.BG.BGConst as bgc
import SpectrumCommon.Const.BG.BGTags as bgmvt
import SpectrumCommon.Modvars.BG.BGReadDatabases as bgrd

def init_modvars(country_ID, first_year, final_year, extra):
    '''
        Initializes Budget Mapping MVs for new IHT projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''

    # Needed to initialize IC indicators 
    deliv_chan_records_mv = extra[bgcl.BG_IH_DELIV_CHAN_RECORDS_TAG]
    interv_records_mv = extra[bgcl.BG_IC_INTERV_RECORDS_TAG]
    drug_supply_records_mv = extra[bgcl.BG_IC_DRUG_SUPPLY_RECORDS_TAG]
    facility_records_mv = extra[bgcl.BG_IS_FACILITY_TYPE_RECORDS_TAG]
    staff_types_mv = extra[bgcl.BG_HW_STAFF_TYPE_RECORDS_TAG]

    # Needed for various costing modes.
    IHT_costing_on = extra[gbc.GB_IHT_ON] if gbc.GB_IHT_ON in extra else False
    TB_costing_on = extra[gbc.GB_TB_COSTING_ON] if gbc.GB_TB_COSTING_ON in extra else False
    LiST_costing_on = extra[gbc.GB_LIST_COSTING_ON] if gbc.GB_LIST_COSTING_ON in extra else False
    costing_mode_mod_vars = {
        pjmvt.GB_IHTOnTag       : IHT_costing_on,
        pjmvt.GB_TBCostingOnTag : TB_costing_on,
        pjmvt.GB_LCOnTag        : LiST_costing_on,
    }
    costing_mode = gbu.get_costing_mode(costing_mode_mod_vars)
    # active_modules = extra[gbc.GB_MODULES]

    # file_list = GB_file_list(environment, gbc.GB_IC_CONTAINER, '')
    # for i, file_name in enumerate(file_list):
    #     log(file_name)

    num_years_inputs = final_year - first_year + 1 
    # num_years_outputs = final_year - first_year + 1
    # num_deliv_chan_IC = iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED + icc.IC_NUM_DELIV_CHANS
    # num_prog_areas = icmvul.IC_get_IH_num_prog_areas(prog_area_records_mv)

    #######################################################################################################
    #                                                                                                     #
    #                                       Inputs                                                        #
    #                                                                                                     #
    #######################################################################################################

    budget_DB = bgrd.BG_load_budget_DB()
    fund_framework_DB = bgrd.BG_load_funding_framework_DB()
    fund_source_DB = bgrd.BG_load_funding_source_DB()

    value_bool_False = False
    value_bool_True = True
    value_int = 0
    value_flt = 0.0
    value_date = date.today().strftime("%B %d, %Y"),   
    value_list = []

    #######################################################################################################
    #                                                                                                     #
    #                                       Inputs                                                        #
    #                                                                                                     #
    #######################################################################################################

    fund_framework_records = bgrd.BG_init_fund_framework_records(fund_framework_DB, costing_mode)
    fund_source_records = bgrd.BG_init_fund_source_records(fund_source_DB, costing_mode)

    budget_records = []
    budget_cat_records = []
    ind_records_IC = []
    ind_records_PC = []
    ind_records_IS = []
    ind_records_HW = []
    ind_records_HS = []
    ind_records_GV = []
    ind_records_HF = []

    # Put back in if for some reason we want to load certain budgets (or all default budgets; hopefully not) by default.
    #  = bgrd.BG_init_budgets_complete(
    #     costing_mode, IHT_plan_type, num_years_inputs, fund_framework_records, fund_source_records, deliv_chan_records_mv, 
    #     interv_records_mv, drug_supply_records_mv)

    # Meta data
    default_budget_list_meta = bgrd.BG_init_default_budget_list_meta(budget_DB, costing_mode)
    default_fund_fw_list_meta = bgrd.BG_init_default_fund_framework_list_meta(fund_framework_DB, costing_mode)

    mod_vars_rec = {
        # Inputs / pre-result / calculated
		
        bgmvt.BG_VERSION_TAG                              : bgc.BG_FILE_VERSION_1_0,
        bgmvt.BG_BUDGET_RECORDS_TAG                       : budget_records,
        bgmvt.BG_INDICATOR_RECORDS_IC_TAG                 : ind_records_IC,
        bgmvt.BG_INDICATOR_RECORDS_PC_TAG                 : ind_records_PC,
        bgmvt.BG_INDICATOR_RECORDS_IS_TAG                 : ind_records_IS,
        bgmvt.BG_INDICATOR_RECORDS_HW_TAG                 : ind_records_HW,
        bgmvt.BG_INDICATOR_RECORDS_HS_TAG                 : ind_records_HS,
        bgmvt.BG_INDICATOR_RECORDS_GV_TAG                 : ind_records_GV,
        bgmvt.BG_INDICATOR_RECORDS_HF_TAG                 : ind_records_HF,
        bgmvt.BG_BUDGET_CAT_RECORDS_TAG                   : budget_cat_records,
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_IC_TAG          : [],
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_PC_TAG          : [],
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_IS_TAG          : [],
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_HW_TAG          : [],
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_HS_TAG          : [],
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_GV_TAG          : [],
        bgmvt.BG_BUDGET_CAT_SHARE_RECORDS_HF_TAG          : [],
        bgmvt.BG_FUND_FW_RECORDS_TAG                      : fund_framework_records,
        bgmvt.BG_FUND_SOURCE_RECORDS_TAG                  : fund_source_records,
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_IC_TAG         : [],
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_PC_TAG         : [],
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_IS_TAG         : [],  
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_HW_TAG         : [],  
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_HS_TAG         : [],  
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_GV_TAG         : [],
        bgmvt.BG_FUND_SOURCE_SHARE_RECORDS_HF_TAG         : [],  
        bgmvt.BG_USING_SAME_FUND_MAPPINGS_ALL_BUDGETS_TAG : False,
        bgmvt.BG_QUICK_MAP_RECORDS_TAG                    : [],

        #icmvt.IC_DATA_CHANGED_TAG_V1                        : value_bool_False, 

        #icmvt.IC_RES_NUM_SERVICES_BY_DELIV_CHAN_TAG_V1      : icmvd.IC_init_interv_deliv_chan_all_year_arr(num_intervs, num_deliv_chan_IC, num_years_outputs, 0.0).tolist(),
        
        bgmvt.BG_RES_TOTAL_COSTS_BY_BUDGET_CAT_TAG             : [],
        bgmvt.BG_RES_TOTAL_COSTS_BY_FUND_SOURCE_TAG            : [],
        bgmvt.BG_RES_TOTAL_COSTS_BY_BUDGET_CAT_FUND_SOURCE_TAG : [],

        bgmvt.BG_DEFAULT_BUDGET_LIST_META_TAG             : default_budget_list_meta,
        bgmvt.BG_DEFAULT_FUND_FW_LIST_META_TAG            : default_fund_fw_list_meta,
    }

    #Debugging
    #with open('C:\Proj\BGSampleFile.JSON', 'w') as f:
    #    json.dump(mod_vars_dict, f)

    return mod_vars_rec